"""Data resource lib."""
from __future__ import unicode_literals
import os
import codecs
import base64
from wx.lib.embeddedimage import PyEmbeddedImage
from ... import util

RESOURCE_PATH = os.path.abspath(os.path.dirname(__file__))


def get_file(file_name, raw=False):
    """Get the data file."""

    text = b'' if raw else ''
    resource = os.path.join(RESOURCE_PATH, file_name)
    if os.path.exists(resource):
        try:
            if raw:
                with open(resource, 'rb') as f:
                    text = f.read()
            else:
                with codecs.open(resource, 'r', encoding='utf-8') as f:
                    text = f.read()
        except Exception:
            pass
    return text


def get_image(file_name, b64=False):
    """Get the image as a PyEmbeddedImage."""
    icon = b''
    resource = os.path.join(RESOURCE_PATH, file_name)
    if os.path.exists(resource):
        try:
            with open(resource, "rb") as f:
                icon = base64.b64encode(f.read())
        except Exception:
            pass
    return PyEmbeddedImage(icon) if not b64 else util.to_ustr(icon)


def get_bitmap(file_name):
    """
    Get bitmap.

    For retina, we provide images that are 2X size.
    We can't detect retina yet, so we use 2X for non retina as well.
    This works fine for OSX as it seems to scale the images to fit the sizer
    as long as you set the height and width of the bitmap object (not the actual data)
    to half size.

    Windows and Linux do not auto scale, so we have to actually scale them.
    In the future, we should load normal sizes for Windows/Linux (and OSX non retina if we can detect retina),
    and load a separate 2X size for OSX retina.  But this is an okay work around for now.
    """

    image = get_image(file_name).GetImage()
    if util.platform() == "osx":
        bm = image.ConvertToBitmap()
        bm.SetSize(
            (
                int(bm.GetWidth() / 2),
                int(bm.GetHeight() / 2)
            )
        )
    else:
        scaled = image.Rescale(
            int(image.GetWidth() / 2),
            int(image.GetHeight() / 2)
        )
        bm = scaled.ConvertToBitmap()

    return bm



"""Global editor dialog."""
from __future__ import unicode_literals
from . import basic_dialogs
from . import gui
from . import settings_key_bindings
from .. import util


class GlobalCssEditor(gui.GlobalSettingCss, settings_key_bindings.SettingsKeyBindings):
    """GlobalCssEditor."""

    def __init__(self, parent, current_entries, name, value, insert=False):
        """Initialize."""

        super(GlobalCssEditor, self).__init__(parent)
        if util.platform() == "windows":
            self.SetDoubleBuffered(True)
        self.setup_keybindings()
        self.Fit()
        size = self.GetSize()
        self.SetMinSize(size)
        size.Set(-1, size[1])
        self.obj_key = name
        self.obj_val = value
        self.apply_settings = False
        self.entries = current_entries
        self.current_name = name
        self.valid = True

        self.m_name_textbox.SetValue(self.obj_key)
        self.m_value_textbox.WriteText(self.obj_val)

    def is_name_valid(self):
        """Check if name is valid."""

        valid = True
        name = self.m_name_textbox.GetValue()
        if name != self.current_name:
            for k in self.entries:
                if name == k:
                    valid = False
                    break
        return valid

    def on_global_name_blur(self, event):
        """Handle global name blur event."""

        if not self.is_name_valid():
            basic_dialogs.errormsg(
                "Key name \"%s\" already exists in global settings. "
                "Please use a different name." % self.m_name_textbox.GetValue()
            )
            self.m_name_textbox.SetValue(self.current_name)
        else:
            self.current_name = self.m_name_textbox.GetValue()

    def on_apply_button_click(self, event):
        """Handle apply button click event."""

        self.m_apply_button.SetFocus()
        if self.is_name_valid():
            self.apply_settings = True
            self.Close()
        else:
            basic_dialogs.errormsg(
                "Key name \"%s\" already exists in global settings. "
                "Please use a different name." % self.m_name_textbox.GetValue()
            )
            self.m_name_textbox.SetValue(self.current_name)

    def on_close(self, event):
        """Handle set color close event."""

        self.obj_key = self.m_name_textbox.GetValue()
        self.current_name = self.obj_key

        if self.apply_settings:
            self.obj_val = self.m_value_textbox.GetValue()
            self.Parent.set_global_object(self.obj_key, self.obj_val)

        event.Skip()



# -*- coding: utf-8 -*- 

###########################################################################
## Python code generated with wxFormBuilder (version Dec 21 2016)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx
import wx.adv
import wx.xrc
import wx.grid

###########################################################################
## Class EditorFrame
###########################################################################

class EditorFrame ( wx.Frame ):
	
	def __init__( self, parent ):
		wx.Frame.__init__ ( self, parent, id = wx.ID_ANY, title = u"Color Scheme Editor", pos = wx.DefaultPosition, size = wx.Size( 599,417 ), style = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )
		
		self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )
		self.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		
		bSizer1 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_main_panel = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		self.m_main_panel.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		self.m_main_panel.SetMinSize( wx.Size( 500,400 ) )
		
		bSizer2 = wx.BoxSizer( wx.VERTICAL )
		
		fgSizer1 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer1.AddGrowableCol( 0 )
		fgSizer1.AddGrowableRow( 1 )
		fgSizer1.SetFlexibleDirection( wx.BOTH )
		fgSizer1.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		bSizer3 = wx.BoxSizer( wx.VERTICAL )
		
		fgSizer2 = wx.FlexGridSizer( 0, 2, 0, 0 )
		fgSizer2.AddGrowableCol( 1 )
		fgSizer2.SetFlexibleDirection( wx.BOTH )
		fgSizer2.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_plist_name_label = wx.StaticText( self.m_main_panel, wx.ID_ANY, u"Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_plist_name_label.Wrap( -1 )
		fgSizer2.Add( self.m_plist_name_label, 0, wx.ALL|wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_plist_name_textbox = wx.TextCtrl( self.m_main_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer2.Add( self.m_plist_name_textbox, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_plist_uuid_label = wx.StaticText( self.m_main_panel, wx.ID_ANY, u"UUID", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_plist_uuid_label.Wrap( -1 )
		fgSizer2.Add( self.m_plist_uuid_label, 0, wx.ALL|wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		fgSizer10 = wx.FlexGridSizer( 0, 2, 0, 0 )
		fgSizer10.AddGrowableCol( 0 )
		fgSizer10.SetFlexibleDirection( wx.BOTH )
		fgSizer10.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_plist_uuid_textbox = wx.TextCtrl( self.m_main_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer10.Add( self.m_plist_uuid_textbox, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_uuid_button = wx.Button( self.m_main_panel, wx.ID_ANY, u"Generate", wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer10.Add( self.m_uuid_button, 0, wx.ALL, 5 )
		
		
		fgSizer2.Add( fgSizer10, 1, wx.EXPAND, 5 )
		
		self.m_search_label = wx.StaticText( self.m_main_panel, wx.ID_ANY, u"Find", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_search_label.Wrap( -1 )
		fgSizer2.Add( self.m_search_label, 0, wx.ALL|wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		fgSizer11 = wx.FlexGridSizer( 0, 3, 0, 0 )
		fgSizer11.AddGrowableCol( 0 )
		fgSizer11.SetFlexibleDirection( wx.BOTH )
		fgSizer11.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_search_panel = wx.TextCtrl( self.m_main_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_PROCESS_ENTER )
		fgSizer11.Add( self.m_search_panel, 1, wx.ALL|wx.EXPAND, 5 )
		
		self.m_button4 = wx.Button( self.m_main_panel, wx.ID_ANY, u"Prev", wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer11.Add( self.m_button4, 0, wx.ALL, 5 )
		
		self.m_button5 = wx.Button( self.m_main_panel, wx.ID_ANY, u"Next", wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer11.Add( self.m_button5, 0, wx.ALL, 5 )
		
		
		fgSizer2.Add( fgSizer11, 1, wx.EXPAND, 5 )
		
		
		bSizer3.Add( fgSizer2, 1, wx.EXPAND, 5 )
		
		
		fgSizer1.Add( bSizer3, 1, wx.EXPAND, 5 )
		
		self.m_plist_notebook = wx.Notebook( self.m_main_panel, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, 0|wx.ALWAYS_SHOW_SB|wx.FULL_REPAINT_ON_RESIZE )
		self.m_plist_notebook.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		
		
		fgSizer1.Add( self.m_plist_notebook, 1, wx.BOTTOM|wx.EXPAND|wx.LEFT|wx.RIGHT, 5 )
		
		
		bSizer2.Add( fgSizer1, 1, wx.EXPAND, 5 )
		
		
		self.m_main_panel.SetSizer( bSizer2 )
		self.m_main_panel.Layout()
		bSizer2.Fit( self.m_main_panel )
		bSizer1.Add( self.m_main_panel, 1, wx.EXPAND |wx.ALL, 0 )
		
		
		self.SetSizer( bSizer1 )
		self.Layout()
		self.m_menubar = wx.MenuBar( 0 )
		self.m_menu_file = wx.Menu()
		self.m_menuitem_open = wx.MenuItem( self.m_menu_file, wx.ID_ANY, u"Open", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu_file.Append( self.m_menuitem_open )
		
		self.m_menuitem_save = wx.MenuItem( self.m_menu_file, wx.ID_ANY, u"Save", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu_file.Append( self.m_menuitem_save )
		
		self.m_menuitem_save_as = wx.MenuItem( self.m_menu_file, wx.ID_ANY, u"Save As", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu_file.Append( self.m_menuitem_save_as )
		
		self.m_menubar.Append( self.m_menu_file, u"File" ) 
		
		self.m_menu_help = wx.Menu()
		self.m_menuitem_keys = wx.MenuItem( self.m_menu_help, wx.ID_ANY, u"Shortcuts", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu_help.Append( self.m_menuitem_keys )
		
		self.m_menuitem_about = wx.MenuItem( self.m_menu_help, wx.ID_ANY, u"About", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu_help.Append( self.m_menuitem_about )
		
		self.m_menubar.Append( self.m_menu_help, u"Help" ) 
		
		self.SetMenuBar( self.m_menubar )
		
		self.m_statusbar = self.CreateStatusBar( 1, wx.STB_SIZEGRIP, wx.ID_ANY )
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.Bind( wx.EVT_CLOSE, self.on_close )
		self.Bind( wx.EVT_KEY_DOWN, self.on_global_key_down )
		self.m_plist_name_textbox.Bind( wx.EVT_KILL_FOCUS, self.on_plist_name_blur )
		self.m_plist_uuid_textbox.Bind( wx.EVT_KILL_FOCUS, self.on_uuid_blur )
		self.m_uuid_button.Bind( wx.EVT_BUTTON, self.on_uuid_button_click )
		self.m_search_panel.Bind( wx.EVT_TEXT, self.on_find )
		self.m_search_panel.Bind( wx.EVT_TEXT_ENTER, self.on_find_finish )
		self.m_button4.Bind( wx.EVT_BUTTON, self.on_prev_find )
		self.m_button5.Bind( wx.EVT_BUTTON, self.on_next_find )
		self.m_plist_notebook.Bind( wx.EVT_SIZE, self.on_plist_notebook_size )
		self.Bind( wx.EVT_MENU, self.on_open_new, id = self.m_menuitem_open.GetId() )
		self.Bind( wx.EVT_MENU, self.on_save, id = self.m_menuitem_save.GetId() )
		self.Bind( wx.EVT_MENU, self.on_save_as, id = self.m_menuitem_save_as.GetId() )
		self.Bind( wx.EVT_MENU, self.on_shortcuts, id = self.m_menuitem_keys.GetId() )
		self.Bind( wx.EVT_MENU, self.on_about, id = self.m_menuitem_about.GetId() )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def on_close( self, event ):
		event.Skip()
	
	def on_global_key_down( self, event ):
		event.Skip()
	
	def on_plist_name_blur( self, event ):
		event.Skip()
	
	def on_uuid_blur( self, event ):
		event.Skip()
	
	def on_uuid_button_click( self, event ):
		event.Skip()
	
	def on_find( self, event ):
		event.Skip()
	
	def on_find_finish( self, event ):
		event.Skip()
	
	def on_prev_find( self, event ):
		event.Skip()
	
	def on_next_find( self, event ):
		event.Skip()
	
	def on_plist_notebook_size( self, event ):
		event.Skip()
	
	def on_open_new( self, event ):
		event.Skip()
	
	def on_save( self, event ):
		event.Skip()
	
	def on_save_as( self, event ):
		event.Skip()
	
	def on_shortcuts( self, event ):
		event.Skip()
	
	def on_about( self, event ):
		event.Skip()
	

###########################################################################
## Class GlobalSettingsPanel
###########################################################################

class GlobalSettingsPanel ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL )
		
		self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		
		bSizer1 = wx.BoxSizer( wx.VERTICAL )
		
		fgSizer1 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer1.AddGrowableCol( 0 )
		fgSizer1.AddGrowableRow( 1 )
		fgSizer1.SetFlexibleDirection( wx.BOTH )
		fgSizer1.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_button_panel = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		self.m_button_panel.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DDKSHADOW ) )
		
		fgSizer2 = wx.FlexGridSizer( 0, 4, 0, 0 )
		fgSizer2.SetFlexibleDirection( wx.BOTH )
		fgSizer2.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_row_add = wx.Button( self.m_button_panel, wx.ID_ANY, u"+", wx.DefaultPosition, wx.Size( 30,23 ), wx.BU_EXACTFIT )
		self.m_row_add.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNTEXT ) )
		
		fgSizer2.Add( self.m_row_add, 0, wx.BOTTOM|wx.LEFT|wx.TOP, 5 )
		
		self.m_row_delete = wx.Button( self.m_button_panel, wx.ID_ANY, u"-", wx.DefaultPosition, wx.Size( 30,23 ), wx.BU_EXACTFIT )
		self.m_row_delete.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNTEXT ) )
		
		fgSizer2.Add( self.m_row_delete, 0, wx.BOTTOM|wx.RIGHT|wx.TOP, 5 )
		
		
		self.m_button_panel.SetSizer( fgSizer2 )
		self.m_button_panel.Layout()
		fgSizer2.Fit( self.m_button_panel )
		fgSizer1.Add( self.m_button_panel, 1, wx.EXPAND, 5 )
		
		self.m_plist_grid = wx.grid.Grid( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, 0 )
		
		# Grid
		self.m_plist_grid.CreateGrid( 0, 2 )
		self.m_plist_grid.EnableEditing( False )
		self.m_plist_grid.EnableGridLines( False )
		self.m_plist_grid.EnableDragGridSize( False )
		self.m_plist_grid.SetMargins( 0, 0 )
		
		# Columns
		self.m_plist_grid.EnableDragColMove( False )
		self.m_plist_grid.EnableDragColSize( False )
		self.m_plist_grid.SetColLabelSize( 30 )
		self.m_plist_grid.SetColLabelValue( 0, u"Name" )
		self.m_plist_grid.SetColLabelValue( 1, u"Value" )
		self.m_plist_grid.SetColLabelAlignment( wx.ALIGN_LEFT, wx.ALIGN_CENTRE )
		
		# Rows
		self.m_plist_grid.EnableDragRowSize( False )
		self.m_plist_grid.SetRowLabelSize( 0 )
		self.m_plist_grid.SetRowLabelAlignment( wx.ALIGN_CENTRE, wx.ALIGN_CENTRE )
		
		# Label Appearance
		
		# Cell Defaults
		self.m_plist_grid.SetDefaultCellAlignment( wx.ALIGN_LEFT, wx.ALIGN_TOP )
		fgSizer1.Add( self.m_plist_grid, 1, wx.EXPAND, 5 )
		
		
		bSizer1.Add( fgSizer1, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( bSizer1 )
		self.Layout()
		
		# Connect Events
		self.m_row_add.Bind( wx.EVT_BUTTON, self.on_row_add_click )
		self.m_row_delete.Bind( wx.EVT_BUTTON, self.on_row_delete_click )
		self.m_plist_grid.Bind( wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.on_edit_cell )
		self.m_plist_grid.Bind( wx.grid.EVT_GRID_LABEL_LEFT_CLICK, self.on_grid_label_left_click )
		self.m_plist_grid.Bind( wx.grid.EVT_GRID_SELECT_CELL, self.on_grid_select_cell )
		self.m_plist_grid.Bind( wx.EVT_KEY_DOWN, self.on_grid_key_down )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def on_row_add_click( self, event ):
		event.Skip()
	
	def on_row_delete_click( self, event ):
		event.Skip()
	
	def on_edit_cell( self, event ):
		event.Skip()
	
	def on_grid_label_left_click( self, event ):
		event.Skip()
	
	def on_grid_select_cell( self, event ):
		event.Skip()
	
	def on_grid_key_down( self, event ):
		event.Skip()
	

###########################################################################
## Class StyleSettingsPanel
###########################################################################

class StyleSettingsPanel ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL )
		
		self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		
		bSizer1 = wx.BoxSizer( wx.VERTICAL )
		
		fgSizer1 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer1.AddGrowableCol( 0 )
		fgSizer1.AddGrowableRow( 1 )
		fgSizer1.SetFlexibleDirection( wx.BOTH )
		fgSizer1.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_button_panel = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		self.m_button_panel.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )
		self.m_button_panel.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DDKSHADOW ) )
		
		fgSizer2 = wx.FlexGridSizer( 0, 4, 0, 0 )
		fgSizer2.SetFlexibleDirection( wx.BOTH )
		fgSizer2.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_row_up = wx.Button( self.m_button_panel, wx.ID_ANY, u"↑", wx.DefaultPosition, wx.Size( 30,23 ), wx.BU_EXACTFIT )
		self.m_row_up.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNTEXT ) )
		
		fgSizer2.Add( self.m_row_up, 0, wx.BOTTOM|wx.LEFT|wx.TOP, 5 )
		
		self.m_row_down = wx.Button( self.m_button_panel, wx.ID_ANY, u"↓", wx.DefaultPosition, wx.Size( 30,23 ), wx.BU_EXACTFIT )
		self.m_row_down.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNTEXT ) )
		
		fgSizer2.Add( self.m_row_down, 0, wx.BOTTOM|wx.RIGHT|wx.TOP, 5 )
		
		self.m_row_add = wx.Button( self.m_button_panel, wx.ID_ANY, u"+", wx.DefaultPosition, wx.Size( 30,23 ), wx.BU_EXACTFIT )
		self.m_row_add.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNTEXT ) )
		
		fgSizer2.Add( self.m_row_add, 0, wx.ALIGN_RIGHT|wx.BOTTOM|wx.LEFT|wx.TOP, 5 )
		
		self.m_row_delete = wx.Button( self.m_button_panel, wx.ID_ANY, u"-", wx.DefaultPosition, wx.Size( 30,23 ), wx.BU_EXACTFIT )
		self.m_row_delete.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNTEXT ) )
		
		fgSizer2.Add( self.m_row_delete, 0, wx.ALIGN_RIGHT|wx.BOTTOM|wx.RIGHT|wx.TOP, 5 )
		
		
		self.m_button_panel.SetSizer( fgSizer2 )
		self.m_button_panel.Layout()
		fgSizer2.Fit( self.m_button_panel )
		fgSizer1.Add( self.m_button_panel, 1, wx.EXPAND, 5 )
		
		self.m_plist_grid = wx.grid.Grid( self, wx.ID_ANY, wx.Point( -1,-1 ), wx.DefaultSize, 0 )
		
		# Grid
		self.m_plist_grid.CreateGrid( 0, 5 )
		self.m_plist_grid.EnableEditing( False )
		self.m_plist_grid.EnableGridLines( False )
		self.m_plist_grid.EnableDragGridSize( False )
		self.m_plist_grid.SetMargins( 0, 0 )
		
		# Columns
		self.m_plist_grid.EnableDragColMove( False )
		self.m_plist_grid.EnableDragColSize( False )
		self.m_plist_grid.SetColLabelSize( 30 )
		self.m_plist_grid.SetColLabelValue( 0, u"Name" )
		self.m_plist_grid.SetColLabelValue( 1, u"Foreground" )
		self.m_plist_grid.SetColLabelValue( 2, u"Background" )
		self.m_plist_grid.SetColLabelValue( 3, u"Font Style" )
		self.m_plist_grid.SetColLabelValue( 4, u"Scope" )
		self.m_plist_grid.SetColLabelAlignment( wx.ALIGN_LEFT, wx.ALIGN_CENTRE )
		
		# Rows
		self.m_plist_grid.AutoSizeRows()
		self.m_plist_grid.EnableDragRowSize( False )
		self.m_plist_grid.SetRowLabelSize( 0 )
		self.m_plist_grid.SetRowLabelAlignment( wx.ALIGN_RIGHT, wx.ALIGN_CENTRE )
		
		# Label Appearance
		
		# Cell Defaults
		self.m_plist_grid.SetDefaultCellAlignment( wx.ALIGN_LEFT, wx.ALIGN_TOP )
		self.m_plist_grid.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )
		self.m_plist_grid.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )
		
		fgSizer1.Add( self.m_plist_grid, 1, wx.EXPAND, 5 )
		
		
		bSizer1.Add( fgSizer1, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( bSizer1 )
		self.Layout()
		
		# Connect Events
		self.m_row_up.Bind( wx.EVT_BUTTON, self.on_row_up_click )
		self.m_row_down.Bind( wx.EVT_BUTTON, self.on_row_down_click )
		self.m_row_add.Bind( wx.EVT_BUTTON, self.on_row_add_click )
		self.m_row_delete.Bind( wx.EVT_BUTTON, self.on_row_delete_click )
		self.m_plist_grid.Bind( wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.on_edit_cell )
		self.m_plist_grid.Bind( wx.grid.EVT_GRID_LABEL_LEFT_CLICK, self.on_grid_label_left_click )
		self.m_plist_grid.Bind( wx.grid.EVT_GRID_SELECT_CELL, self.on_grid_select_cell )
		self.m_plist_grid.Bind( wx.EVT_KEY_DOWN, self.on_grid_key_down )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def on_row_up_click( self, event ):
		event.Skip()
	
	def on_row_down_click( self, event ):
		event.Skip()
	
	def on_row_add_click( self, event ):
		event.Skip()
	
	def on_row_delete_click( self, event ):
		event.Skip()
	
	def on_edit_cell( self, event ):
		event.Skip()
	
	def on_grid_label_left_click( self, event ):
		event.Skip()
	
	def on_grid_select_cell( self, event ):
		event.Skip()
	
	def on_grid_key_down( self, event ):
		event.Skip()
	

###########################################################################
## Class GlobalSetting
###########################################################################

class GlobalSetting ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Global Setting", pos = wx.DefaultPosition, size = wx.Size( 500,150 ), style = wx.DEFAULT_DIALOG_STYLE|wx.MAXIMIZE_BOX|wx.RESIZE_BORDER )
		
		self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )
		self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		
		bSizer1 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_global_setting_panel = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		bSizer2 = wx.BoxSizer( wx.VERTICAL )
		
		fgSizer1 = wx.FlexGridSizer( 0, 2, 0, 0 )
		fgSizer1.AddGrowableCol( 1 )
		fgSizer1.SetFlexibleDirection( wx.BOTH )
		fgSizer1.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_name_label = wx.StaticText( self.m_global_setting_panel, wx.ID_ANY, u"Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_name_label.Wrap( -1 )
		self.m_name_label.SetFont( wx.Font( wx.NORMAL_FONT.GetPointSize(), wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, wx.EmptyString ) )
		
		fgSizer1.Add( self.m_name_label, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.m_name_textbox = wx.TextCtrl( self.m_global_setting_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer1.Add( self.m_name_textbox, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		fgSizer1.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		fgSizer2 = wx.FlexGridSizer( 0, 4, 0, 0 )
		fgSizer2.AddGrowableCol( 2 )
		fgSizer2.SetFlexibleDirection( wx.BOTH )
		fgSizer2.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_color_checkbox = wx.CheckBox( self.m_global_setting_panel, wx.ID_ANY, u"Color", wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer2.Add( self.m_color_checkbox, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_color_picker = wx.Panel( self.m_global_setting_panel, wx.ID_ANY, wx.DefaultPosition, wx.Size( 30,-1 ), wx.SUNKEN_BORDER|wx.TAB_TRAVERSAL )
		fgSizer2.Add( self.m_color_picker, 1, wx.EXPAND |wx.ALL, 5 )
		
		self.m_value_textbox = wx.TextCtrl( self.m_global_setting_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer2.Add( self.m_value_textbox, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		fgSizer1.Add( fgSizer2, 1, wx.EXPAND, 5 )
		
		
		fgSizer1.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		fgSizer3 = wx.FlexGridSizer( 0, 3, 0, 0 )
		fgSizer3.AddGrowableCol( 1 )
		fgSizer3.SetFlexibleDirection( wx.BOTH )
		fgSizer3.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		
		fgSizer3.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		self.m_apply_button = wx.Button( self.m_global_setting_panel, wx.ID_ANY, u"Apply", wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer3.Add( self.m_apply_button, 0, wx.ALIGN_RIGHT|wx.ALL, 5 )
		
		
		fgSizer1.Add( fgSizer3, 1, wx.EXPAND, 5 )
		
		
		bSizer2.Add( fgSizer1, 1, wx.EXPAND, 5 )
		
		
		self.m_global_setting_panel.SetSizer( bSizer2 )
		self.m_global_setting_panel.Layout()
		bSizer2.Fit( self.m_global_setting_panel )
		bSizer1.Add( self.m_global_setting_panel, 1, wx.EXPAND |wx.ALL, 5 )
		
		
		self.SetSizer( bSizer1 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.Bind( wx.EVT_CLOSE, self.on_set_color_close )
		self.m_name_textbox.Bind( wx.EVT_KILL_FOCUS, self.on_global_name_blur )
		self.m_color_checkbox.Bind( wx.EVT_CHECKBOX, self.on_global_checkbox )
		self.m_color_picker.Bind( wx.EVT_LEFT_DOWN, self.on_color_button_click )
		self.m_value_textbox.Bind( wx.EVT_KILL_FOCUS, self.on_color_blur )
		self.m_value_textbox.Bind( wx.EVT_SET_FOCUS, self.on_color_focus )
		self.m_value_textbox.Bind( wx.EVT_TEXT, self.on_color_change )
		self.m_apply_button.Bind( wx.EVT_BUTTON, self.on_apply_button_click )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def on_set_color_close( self, event ):
		event.Skip()
	
	def on_global_name_blur( self, event ):
		event.Skip()
	
	def on_global_checkbox( self, event ):
		event.Skip()
	
	def on_color_button_click( self, event ):
		event.Skip()
	
	def on_color_blur( self, event ):
		event.Skip()
	
	def on_color_focus( self, event ):
		event.Skip()
	
	def on_color_change( self, event ):
		event.Skip()
	
	def on_apply_button_click( self, event ):
		event.Skip()
	

###########################################################################
## Class GlobalSettingCss
###########################################################################

class GlobalSettingCss ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Global Setting CSS", pos = wx.DefaultPosition, size = wx.Size( 300,400 ), style = wx.DEFAULT_DIALOG_STYLE|wx.MAXIMIZE_BOX|wx.RESIZE_BORDER )
		
		self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )
		self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		
		bSizer1 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_global_setting_panel = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		bSizer2 = wx.BoxSizer( wx.VERTICAL )
		
		fgSizer1 = wx.FlexGridSizer( 0, 2, 0, 0 )
		fgSizer1.AddGrowableCol( 1 )
		fgSizer1.AddGrowableRow( 1 )
		fgSizer1.SetFlexibleDirection( wx.BOTH )
		fgSizer1.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_name_label = wx.StaticText( self.m_global_setting_panel, wx.ID_ANY, u"Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_name_label.Wrap( -1 )
		self.m_name_label.SetFont( wx.Font( wx.NORMAL_FONT.GetPointSize(), wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, wx.EmptyString ) )
		
		fgSizer1.Add( self.m_name_label, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.m_name_textbox = wx.TextCtrl( self.m_global_setting_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
		fgSizer1.Add( self.m_name_textbox, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		fgSizer1.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		self.m_value_textbox = wx.TextCtrl( self.m_global_setting_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_MULTILINE|wx.TE_PROCESS_TAB )
		self.m_value_textbox.SetMinSize( wx.Size( 200,200 ) )
		
		fgSizer1.Add( self.m_value_textbox, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		fgSizer1.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		fgSizer3 = wx.FlexGridSizer( 0, 3, 0, 0 )
		fgSizer3.AddGrowableCol( 1 )
		fgSizer3.SetFlexibleDirection( wx.BOTH )
		fgSizer3.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		
		fgSizer3.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		self.m_apply_button = wx.Button( self.m_global_setting_panel, wx.ID_ANY, u"Apply", wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer3.Add( self.m_apply_button, 0, wx.ALIGN_RIGHT|wx.ALL, 5 )
		
		
		fgSizer1.Add( fgSizer3, 1, wx.EXPAND, 5 )
		
		
		bSizer2.Add( fgSizer1, 1, wx.EXPAND, 5 )
		
		
		self.m_global_setting_panel.SetSizer( bSizer2 )
		self.m_global_setting_panel.Layout()
		bSizer2.Fit( self.m_global_setting_panel )
		bSizer1.Add( self.m_global_setting_panel, 1, wx.EXPAND |wx.ALL, 5 )
		
		
		self.SetSizer( bSizer1 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.Bind( wx.EVT_CLOSE, self.on_close )
		self.m_name_textbox.Bind( wx.EVT_KILL_FOCUS, self.on_global_name_blur )
		self.m_value_textbox.Bind( wx.EVT_KILL_FOCUS, self.on_color_blur )
		self.m_value_textbox.Bind( wx.EVT_SET_FOCUS, self.on_color_focus )
		self.m_apply_button.Bind( wx.EVT_BUTTON, self.on_apply_button_click )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def on_close( self, event ):
		event.Skip()
	
	def on_global_name_blur( self, event ):
		event.Skip()
	
	def on_color_blur( self, event ):
		event.Skip()
	
	def on_color_focus( self, event ):
		event.Skip()
	
	def on_apply_button_click( self, event ):
		event.Skip()
	

###########################################################################
## Class ColorSetting
###########################################################################

class ColorSetting ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Color Setting", pos = wx.DefaultPosition, size = wx.Size( 500,200 ), style = wx.DEFAULT_DIALOG_STYLE|wx.MAXIMIZE_BOX|wx.RESIZE_BORDER )
		
		self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )
		self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		
		bSizer1 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_color_setting_panel = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		self.m_color_setting_panel.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		
		bSizer2 = wx.BoxSizer( wx.VERTICAL )
		
		fgSizer1 = wx.FlexGridSizer( 0, 2, 0, 0 )
		fgSizer1.AddGrowableCol( 1 )
		fgSizer1.SetFlexibleDirection( wx.BOTH )
		fgSizer1.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_name_label = wx.StaticText( self.m_color_setting_panel, wx.ID_ANY, u"Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_name_label.Wrap( -1 )
		self.m_name_label.SetFont( wx.Font( wx.NORMAL_FONT.GetPointSize(), wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, wx.EmptyString ) )
		
		fgSizer1.Add( self.m_name_label, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.ALIGN_RIGHT, 5 )
		
		self.m_name_textbox = wx.TextCtrl( self.m_color_setting_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer1.Add( self.m_name_textbox, 1, wx.ALL|wx.EXPAND, 5 )
		
		self.m_scope_label = wx.StaticText( self.m_color_setting_panel, wx.ID_ANY, u"Scope", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_scope_label.Wrap( -1 )
		self.m_scope_label.SetFont( wx.Font( wx.NORMAL_FONT.GetPointSize(), wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, wx.EmptyString ) )
		
		fgSizer1.Add( self.m_scope_label, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.ALIGN_RIGHT, 5 )
		
		self.m_scope_textbox = wx.TextCtrl( self.m_color_setting_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer1.Add( self.m_scope_textbox, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		fgSizer1.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		fgSizer2 = wx.FlexGridSizer( 0, 2, 0, 0 )
		fgSizer2.AddGrowableCol( 1 )
		fgSizer2.SetFlexibleDirection( wx.BOTH )
		fgSizer2.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		gSizer1 = wx.GridSizer( 0, 3, 0, 0 )
		
		self.m_bold_checkbox = wx.CheckBox( self.m_color_setting_panel, wx.ID_ANY, u"bold", wx.DefaultPosition, wx.DefaultSize, 0 )
		gSizer1.Add( self.m_bold_checkbox, 0, wx.ALL, 5 )
		
		self.m_italic_checkbox = wx.CheckBox( self.m_color_setting_panel, wx.ID_ANY, u"italic", wx.DefaultPosition, wx.DefaultSize, 0 )
		gSizer1.Add( self.m_italic_checkbox, 0, wx.ALL, 5 )
		
		self.m_underline_checkbox = wx.CheckBox( self.m_color_setting_panel, wx.ID_ANY, u"underline", wx.DefaultPosition, wx.DefaultSize, 0 )
		gSizer1.Add( self.m_underline_checkbox, 0, wx.ALL, 5 )
		
		
		fgSizer2.Add( gSizer1, 1, wx.EXPAND, 5 )
		
		
		fgSizer2.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		
		fgSizer1.Add( fgSizer2, 1, wx.EXPAND, 5 )
		
		
		fgSizer1.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		fgSizer3 = wx.FlexGridSizer( 0, 5, 0, 0 )
		fgSizer3.AddGrowableCol( 4 )
		fgSizer3.SetFlexibleDirection( wx.BOTH )
		fgSizer3.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_foreground_picker = wx.Panel( self.m_color_setting_panel, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.SUNKEN_BORDER|wx.TAB_TRAVERSAL )
		bSizer3 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_foreground_button_label = wx.StaticText( self.m_foreground_picker, wx.ID_ANY, u"foreground", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_foreground_button_label.Wrap( -1 )
		bSizer3.Add( self.m_foreground_button_label, 0, wx.ALIGN_CENTER|wx.ALL, 3 )
		
		
		self.m_foreground_picker.SetSizer( bSizer3 )
		self.m_foreground_picker.Layout()
		bSizer3.Fit( self.m_foreground_picker )
		fgSizer3.Add( self.m_foreground_picker, 1, wx.ALL|wx.EXPAND, 5 )
		
		self.m_foreground_textbox = wx.TextCtrl( self.m_color_setting_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer3.Add( self.m_foreground_textbox, 0, wx.ALIGN_CENTER|wx.ALL, 5 )
		
		self.m_background_picker = wx.Panel( self.m_color_setting_panel, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.SUNKEN_BORDER|wx.TAB_TRAVERSAL )
		bSizer4 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_background_button_label = wx.StaticText( self.m_background_picker, wx.ID_ANY, u"background", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_background_button_label.Wrap( -1 )
		bSizer4.Add( self.m_background_button_label, 0, wx.ALL, 3 )
		
		
		self.m_background_picker.SetSizer( bSizer4 )
		self.m_background_picker.Layout()
		bSizer4.Fit( self.m_background_picker )
		fgSizer3.Add( self.m_background_picker, 1, wx.EXPAND |wx.ALL, 5 )
		
		self.m_background_textbox = wx.TextCtrl( self.m_color_setting_panel, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer3.Add( self.m_background_textbox, 0, wx.ALIGN_CENTER|wx.ALL, 5 )
		
		
		fgSizer3.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		
		fgSizer1.Add( fgSizer3, 1, wx.EXPAND, 5 )
		
		
		fgSizer1.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		fgSizer4 = wx.FlexGridSizer( 0, 3, 0, 0 )
		fgSizer4.AddGrowableCol( 1 )
		fgSizer4.SetFlexibleDirection( wx.BOTH )
		fgSizer4.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		
		fgSizer4.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		self.m_apply_button = wx.Button( self.m_color_setting_panel, wx.ID_ANY, u"Apply", wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer4.Add( self.m_apply_button, 0, wx.ALIGN_RIGHT|wx.ALL, 5 )
		
		
		fgSizer1.Add( fgSizer4, 1, wx.EXPAND, 5 )
		
		
		bSizer2.Add( fgSizer1, 1, wx.EXPAND, 5 )
		
		
		self.m_color_setting_panel.SetSizer( bSizer2 )
		self.m_color_setting_panel.Layout()
		bSizer2.Fit( self.m_color_setting_panel )
		bSizer1.Add( self.m_color_setting_panel, 1, wx.EXPAND |wx.ALL, 5 )
		
		
		self.SetSizer( bSizer1 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.Bind( wx.EVT_CLOSE, self.on_set_color_close )
		self.m_foreground_button_label.Bind( wx.EVT_LEFT_DOWN, self.on_foreground_button_click )
		self.m_foreground_textbox.Bind( wx.EVT_KILL_FOCUS, self.on_foreground_blur )
		self.m_foreground_textbox.Bind( wx.EVT_SET_FOCUS, self.on_foreground_focus )
		self.m_foreground_textbox.Bind( wx.EVT_TEXT, self.on_foreground_change )
		self.m_background_button_label.Bind( wx.EVT_LEFT_DOWN, self.on_background_button_click )
		self.m_background_textbox.Bind( wx.EVT_KILL_FOCUS, self.on_background_blur )
		self.m_background_textbox.Bind( wx.EVT_SET_FOCUS, self.on_background_focus )
		self.m_background_textbox.Bind( wx.EVT_TEXT, self.on_background_change )
		self.m_apply_button.Bind( wx.EVT_BUTTON, self.on_apply_button_click )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def on_set_color_close( self, event ):
		event.Skip()
	
	def on_foreground_button_click( self, event ):
		event.Skip()
	
	def on_foreground_blur( self, event ):
		event.Skip()
	
	def on_foreground_focus( self, event ):
		event.Skip()
	
	def on_foreground_change( self, event ):
		event.Skip()
	
	def on_background_button_click( self, event ):
		event.Skip()
	
	def on_background_blur( self, event ):
		event.Skip()
	
	def on_background_focus( self, event ):
		event.Skip()
	
	def on_background_change( self, event ):
		event.Skip()
	
	def on_apply_button_click( self, event ):
		event.Skip()
	

###########################################################################
## Class AboutDialog
###########################################################################

class AboutDialog ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"About", pos = wx.DefaultPosition, size = wx.DefaultSize, style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )
		
		bSizer12 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_about_panel = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		self.m_about_panel.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BTNFACE ) )
		
		fgSizer33 = wx.FlexGridSizer( 0, 2, 0, 0 )
		fgSizer33.AddGrowableCol( 1 )
		fgSizer33.SetFlexibleDirection( wx.BOTH )
		fgSizer33.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_bitmap = wx.StaticBitmap( self.m_about_panel, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.Size( 64,64 ), 0 )
		fgSizer33.Add( self.m_bitmap, 0, wx.ALL, 5 )
		
		fgSizer34 = wx.FlexGridSizer( 0, 1, 0, 0 )
		fgSizer34.AddGrowableCol( 0 )
		fgSizer34.SetFlexibleDirection( wx.BOTH )
		fgSizer34.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_app_label = wx.StaticText( self.m_about_panel, wx.ID_ANY, u"SubClrSchm", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTRE )
		self.m_app_label.Wrap( -1 )
		self.m_app_label.SetFont( wx.Font( 20, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, False, wx.EmptyString ) )
		
		fgSizer34.Add( self.m_app_label, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_version_label = wx.StaticText( self.m_about_panel, wx.ID_ANY, u"Version", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTRE )
		self.m_version_label.Wrap( -1 )
		fgSizer34.Add( self.m_version_label, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_dev_toggle = wx.ToggleButton( self.m_about_panel, wx.ID_ANY, u"Contact >>", wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer34.Add( self.m_dev_toggle, 0, wx.ALL, 5 )
		
		self.m_staticline4 = wx.StaticLine( self.m_about_panel, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		fgSizer34.Add( self.m_staticline4, 0, wx.EXPAND |wx.ALL, 5 )
		
		self.m_developers_label = wx.StaticText( self.m_about_panel, wx.ID_ANY, u"Dev", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTRE )
		self.m_developers_label.Wrap( -1 )
		self.m_developers_label.Hide()
		
		fgSizer34.Add( self.m_developers_label, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		fgSizer33.Add( fgSizer34, 1, wx.EXPAND, 5 )
		
		
		fgSizer33.Add( ( 64, 0), 1, wx.EXPAND, 5 )
		
		
		fgSizer33.Add( ( 0, 0), 1, wx.EXPAND, 5 )
		
		
		self.m_about_panel.SetSizer( fgSizer33 )
		self.m_about_panel.Layout()
		fgSizer33.Fit( self.m_about_panel )
		bSizer12.Add( self.m_about_panel, 1, wx.EXPAND |wx.ALL, 5 )
		
		
		self.SetSizer( bSizer12 )
		self.Layout()
		bSizer12.Fit( self )
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_dev_toggle.Bind( wx.EVT_TOGGLEBUTTON, self.on_toggle )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def on_toggle( self, event ):
		event.Skip()



"""Sublime Text Color Scheme Editor."""
from __future__ import unicode_literals
import argparse
import os
import sys
from .lib import __meta__
from .lib.gui import custom_app
from .lib.gui import subclrschm_app
from .lib import util


def parse_arguments(script):
    """Parse the command arguments."""

    parser = argparse.ArgumentParser(
        prog='subclrschm',
        description='Sublime Color Scheme Editor - Edit Sublime Color Scheme'
    )
    # Flag arguments
    parser.add_argument('--version', action='version', version=('%(prog)s ' + __meta__.__version__))
    parser.add_argument('--debug', action='store_true', default=False, help=argparse.SUPPRESS)
    parser.add_argument('--no-redirect', action='store_true', default=False, help=argparse.SUPPRESS)
    parser.add_argument('--multi-instance', '-m', action='store_true', default=False, help="Allow multiple instances")
    parser.add_argument(
        '--log', '-l', nargs='?', default='',
        help="Absolute path to directory to store log file"
    )
    parser.add_argument('--live-save', '-L', action='store_true', default=False, help="Enable live save.")
    # Mutually exclusinve flags
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--select', '-s', action='store_true', default=False, help="Prompt for theme selection")
    group.add_argument('--new', '-n', action='store_true', default=False, help="Open prompting for new theme to create")
    # Positional
    parser.add_argument('file', nargs='?', default=None, help='Theme file')
    return parser.parse_args(util.to_unicode_argv()[1:])


def get_log_location():
    """Get log location."""

    platform = util.platform()

    if platform == "windows":
        folder = os.path.expanduser("~\\.subclrschm")
    elif platform == "osx":
        folder = os.path.expanduser("~/.subclrschm")
    elif platform == "linux":
        folder = os.path.expanduser("~/.config/subclrschm")

    if not os.path.exists(folder):
        os.mkdir(folder)

    return folder


def run():
    """Run the app."""

    script = os.path.dirname(os.path.abspath(sys.argv[0]))
    args = parse_arguments(script)

    if not args.log:
        args.log = get_log_location()

    if os.path.exists(args.log):
        args.log = os.path.join(os.path.normpath(args.log), 'subclrschm.log')

    custom_app.init_app_log(args.log)
    if args.debug:
        custom_app.set_debug_mode(True)

    app = custom_app.CustomApp(
        redirect=not args.no_redirect,
        single_instance_name="subclrschm" if not args.multi_instance else None
    )

    if args.multi_instance or app.is_instance_okay():

        action = ""
        if args.select:
            action = "select"
        elif args.new:
            action = "new"

        main_win = subclrschm_app.Editor(
            None,
            live_save=args.live_save,
            debugging=args.debug
        )

        main_win.Show()
        main_win.init_frame(args.file, action)

        if main_win.is_ready():
            app.MainLoop()
    return 0


def main():
    """Main entry point."""

    sys.exit(run())


if __name__ == "__main__":
    main()



"""Global settigns panel."""
from __future__ import unicode_literals
import wx
from . import gui
from . import grid_helper
from . import global_setting_dialog
from . import global_setting_css_dialog
from . import settings_codes as sc
from ..rgba import RGBA
from ..x11colors import name2hex
from .. import util


class GlobalSettings(gui.GlobalSettingsPanel, grid_helper.GridHelper):
    """GlobalSettings."""

    def __init__(self, parent, scheme, update, reshow):
        """Initialize."""

        super(GlobalSettings, self).__init__(parent)
        if util.platform() == "windows":
            self.SetDoubleBuffered(False)
        self.setup_keybindings()
        self.parent = parent
        self.m_plist_grid.GetGridWindow().Bind(wx.EVT_MOTION, self.on_mouse_motion)
        self.m_plist_grid.SetDefaultCellBackgroundColour(self.GetBackgroundColour())
        self.read_plist(scheme)
        self.reshow = reshow
        self.update_plist = update

    def read_plist(self, scheme):
        """Read plist to get global settings."""

        color = scheme["settings"][0]["settings"].get("foreground", "#000000").strip()
        if not color.startswith('#'):
            color = name2hex(color)
        foreground = RGBA(color)
        color = scheme["settings"][0]["settings"].get("background", "#FFFFFF").strip()
        if not color.startswith('#'):
            color = name2hex(color)
        background = RGBA(color)
        global BG_COLOR
        BG_COLOR = background
        global FG_COLOR
        FG_COLOR = foreground
        count = 0

        found_popup = False
        found_phantom = False
        for k in sorted(scheme["settings"][0]["settings"].keys()):
            if k == "phantomCss":
                found_phantom = True
            elif k == "popupCss":
                found_popup = True
            v = scheme["settings"][0]["settings"][k].strip()
            self.m_plist_grid.AppendRows(1)
            if not v.startswith('#'):
                color = name2hex(v)
                if color is not None:
                    v = color
            self.update_row(count, k, v)
            count += 1

        # Add phantom and/or popup if not found
        if not found_popup:
            self.m_plist_grid.AppendRows(1)
            self.update_row(count, 'popupCss', '')
            count += 1
        if not found_phantom:
            self.m_plist_grid.AppendRows(1)
            self.update_row(count, 'phantomCss', '')

        self.resize_table()

        self.go_cell(self.m_plist_grid, 0, 0)

    def resize_table(self):
        """Resize teh table."""

        self.m_plist_grid.BeginBatch()
        nb_size = self.parent.GetSize()
        total_size = 0
        for x in range(0, 2):
            self.m_plist_grid.AutoSizeColumn(x)
            total_size += self.m_plist_grid.GetColSize(x)
        delta = nb_size[0] - 20 - total_size
        if delta > 0:
            self.m_plist_grid.SetColSize(1, self.m_plist_grid.GetColSize(1) + delta)
        self.m_plist_grid.EndBatch()

    def update_row(self, count, k, v):
        """Update row."""

        try:
            bg = RGBA(v.strip())
            if k != "background":
                editor = self.GetParent().GetParent().GetParent()
                bg.apply_alpha(editor.m_style_settings.bg_color.get_rgb())
            fg = RGBA("#000000") if bg.get_luminance() > 128 else RGBA("#FFFFFF")
        except:
            bg = RGBA("#FFFFFF")
            fg = RGBA("#000000")

        self.m_plist_grid.SetCellValue(count, 0, k)
        self.m_plist_grid.SetCellValue(count, 1, v)

        b = self.m_plist_grid.GetCellBackgroundColour(count, 0)
        f = self.m_plist_grid.GetCellTextColour(count, 0)

        b.Set(bg.r, bg.g, bg.b)
        f.Set(fg.r, fg.g, fg.b)

        self.m_plist_grid.SetCellBackgroundColour(count, 0, b)
        self.m_plist_grid.SetCellBackgroundColour(count, 1, b)

        self.m_plist_grid.SetCellTextColour(count, 0, f)
        self.m_plist_grid.SetCellTextColour(count, 1, f)

    def set_object(self, key, value):
        """Set the object."""

        row = self.m_plist_grid.GetGridCursorRow()
        col = self.m_plist_grid.GetGridCursorCol()
        self.update_row(row, key, value)
        self.update_plist(sc.MODIFY, {"table": "global", "index": key, "data": value})
        if key == "background" or key == "foreground":
            self.reshow(row, col)
        self.resize_table()

    def delete_row(self):
        """Delete the row."""

        row = self.m_plist_grid.GetGridCursorRow()
        col = self.m_plist_grid.GetGridCursorCol()
        name = self.m_plist_grid.GetCellValue(row, 0)
        if name in ('popupCss', 'phantomCss'):
            return
        self.m_plist_grid.DeleteRows(row, 1)
        self.m_plist_grid.GetParent().update_plist(sc.DELETE, {"table": "global", "index": name})
        if name == "foreground" or name == "background":
            self.m_plist_grid.GetParent().reshow(row, col)

    def validate_name(self, name):
        """Validate the name."""

        valid = True
        editor = self.GetParent().GetParent().GetParent()
        for k in editor.scheme["settings"][0]["settings"]:
            if name == k:
                valid = False
                break
        return valid

    def insert_row(self):
        """Insert a new row."""

        new_name = "new_item"
        count = 0
        while not self.validate_name(new_name):
            new_name = "new_item_%d" % count
            count += 1

        editor = self.GetParent().GetParent().GetParent()
        diag = global_setting_dialog.GlobalEditor(
            editor,
            editor.scheme["settings"][0]["settings"],
            new_name,
            "nothing",
            insert=True
        )
        diag.ShowModal()
        diag.Destroy()

    def edit_cell(self):
        """Edit the cell."""
        grid = self.m_plist_grid
        row = grid.GetGridCursorRow()
        editor = self.GetParent().GetParent().GetParent()
        name = grid.GetCellValue(row, 0)
        if name in ('popupCss', 'phantomCss'):
            diag = global_setting_css_dialog.GlobalCssEditor(
                editor,
                editor.scheme["settings"][0]["settings"],
                grid.GetCellValue(row, 0),
                grid.GetCellValue(row, 1)
            )
            diag.ShowModal()
        else:
            diag = global_setting_dialog.GlobalEditor(
                editor,
                editor.scheme["settings"][0]["settings"],
                grid.GetCellValue(row, 0),
                grid.GetCellValue(row, 1)
            )
            diag.ShowModal()
        diag.Destroy()

    def on_grid_label_left_click(self, event):
        """Handle grid label left click."""

        return

    def on_mouse_motion(self, event):
        """Handle mouse motion event."""

        self.mouse_motion(event)

    def on_edit_cell(self, event):
        """Handle edit cell event."""

        self.edit_cell()

    def on_grid_key_down(self, event):
        """Handle grid key down event."""

        self.grid_key_down(event)

    def on_grid_select_cell(self, event):
        """Handle grid select cell event."""

        self.grid_select_cell(event)

    def on_row_add_click(self, event):
        """Handle add row click event."""

        self.insert_row()

    def on_row_delete_click(self, event):
        """Handle delete row event."""

        self.delete_row()



"""Style settings panel."""
from __future__ import unicode_literals
import wx
from . import gui
from . import color_setting_dialog
from . import grid_helper
from . import settings_codes as sc
from ..rgba import RGBA
from ..x11colors import name2hex
from .. import util


class StyleSettings(gui.StyleSettingsPanel, grid_helper.GridHelper):
    """Style settings handler."""

    def __init__(self, parent, scheme, update):
        """Initialize."""

        super(StyleSettings, self).__init__(parent)
        if util.platform() == "windows":
            self.SetDoubleBuffered(False)
        self.setup_keybindings()
        self.parent = parent
        self.m_plist_grid.GetGridWindow().Bind(wx.EVT_MOTION, self.on_mouse_motion)
        self.m_plist_grid.SetDefaultCellBackgroundColour(self.GetBackgroundColour())
        self.read_plist(scheme)
        self.update_plist = update

    def read_plist(self, scheme):
        """Read the plist."""

        color = scheme["settings"][0]["settings"].get("foreground", "#000000").strip()
        if not color.startswith('#'):
            color = name2hex(color)
        foreground = RGBA(color)
        color = scheme["settings"][0]["settings"].get("background", "#FFFFFF").strip()
        if not color.startswith('#'):
            color = name2hex(color)
        background = RGBA(color)
        self.bg_color = background
        self.fg_color = foreground
        count = 0

        for s in scheme["settings"]:
            if "name" in s:
                self.m_plist_grid.AppendRows(1)
                self.update_row(count, s)
                count += 1
        self.resize_table()
        self.go_cell(self.m_plist_grid, 0, 0)

    def update_row(self, count, s):
        """Update stye row."""

        self.m_plist_grid.SetCellValue(count, 0, s["name"])
        self.m_plist_grid.SetCellValue(count, 4, s.get("scope", ""))
        settings = s["settings"]
        b = self.m_plist_grid.GetCellBackgroundColour(count, 0)
        if "background" in settings:
            try:
                named_color = name2hex(settings["background"].strip())
                color = named_color if named_color is not None else settings["background"].strip()
                bg = RGBA(color)
                bg.apply_alpha(self.bg_color.get_rgb())
                self.m_plist_grid.SetCellValue(count, 2, color)
            except:
                bg = self.bg_color
                self.m_plist_grid.SetCellValue(count, 2, "")
        else:
            bg = self.bg_color
        b = self.m_plist_grid.GetCellBackgroundColour(count, 0)
        b.Set(bg.r, bg.g, bg.b)
        self.m_plist_grid.SetCellBackgroundColour(count, 0, b)
        self.m_plist_grid.SetCellBackgroundColour(count, 1, b)
        self.m_plist_grid.SetCellBackgroundColour(count, 2, b)
        self.m_plist_grid.SetCellBackgroundColour(count, 3, b)
        self.m_plist_grid.SetCellBackgroundColour(count, 4, b)
        if "foreground" in settings:
            try:
                named_color = name2hex(settings["foreground"].strip())
                color = named_color if named_color is not None else settings["foreground"].strip()
                fg = RGBA(color)
                fg.apply_alpha(self.bg_color.get_rgb())
                self.m_plist_grid.SetCellValue(count, 1, color)
            except:
                fg = self.fg_color
                self.m_plist_grid.SetCellValue(count, 1, "")
        else:
            fg = self.fg_color
        f = self.m_plist_grid.GetCellTextColour(count, 0)
        f.Set(fg.r, fg.g, fg.b)
        self.m_plist_grid.SetCellTextColour(count, 0, f)
        self.m_plist_grid.SetCellTextColour(count, 1, f)
        self.m_plist_grid.SetCellTextColour(count, 2, f)
        self.m_plist_grid.SetCellTextColour(count, 3, f)
        self.m_plist_grid.SetCellTextColour(count, 4, f)

        fs_setting = settings.get("fontStyle", "")
        font_style = []
        for x in fs_setting.split(" "):
            if x in ["bold", "italic", "underline"]:
                font_style.append(x)

        self.m_plist_grid.SetCellValue(count, 3, " ".join(font_style))
        fs = self.m_plist_grid.GetCellFont(count, 0)
        fs.SetWeight(wx.FONTWEIGHT_NORMAL)
        fs.SetStyle(wx.FONTSTYLE_NORMAL)
        fs.SetUnderlined(False)

        if "bold" in font_style:
            fs.SetWeight(wx.FONTWEIGHT_BOLD)

        if "italic" in font_style:
            fs.SetStyle(wx.FONTSTYLE_ITALIC)

        if "underline" in font_style:
            fs.SetUnderlined(True)

        self.m_plist_grid.SetCellFont(count, 0, fs)
        self.m_plist_grid.SetCellFont(count, 1, fs)
        self.m_plist_grid.SetCellFont(count, 2, fs)
        self.m_plist_grid.SetCellFont(count, 3, fs)
        self.m_plist_grid.SetCellFont(count, 4, fs)

    def resize_table(self):
        """Resize the table."""

        self.m_plist_grid.BeginBatch()
        nb_size = self.parent.GetSize()
        total_size = 0
        for x in range(0, 5):
            self.m_plist_grid.AutoSizeColumn(x)
            total_size += self.m_plist_grid.GetColSize(x)
        delta = nb_size[0] - 20 - total_size
        if delta > 0:
            self.m_plist_grid.SetColSize(4, self.m_plist_grid.GetColSize(4) + delta)
        self.m_plist_grid.EndBatch()

    def set_object(self, obj):
        """Set the object."""

        row = self.m_plist_grid.GetGridCursorRow()
        self.update_row(row, obj)
        self.update_plist(sc.MODIFY, {"table": "style", "index": row, "data": obj})
        self.resize_table()

    def edit_cell(self):
        """Handle editting the cell."""

        grid = self.m_plist_grid
        row = grid.GetGridCursorRow()
        editor = self.GetParent().GetParent().GetParent()
        color_setting_dialog.ColorEditor(
            editor,
            {
                "name": grid.GetCellValue(row, 0),
                "scope": grid.GetCellValue(row, 4),
                "settings": {
                    "foreground": grid.GetCellValue(row, 1),
                    "background": grid.GetCellValue(row, 2),
                    "fontStyle": grid.GetCellValue(row, 3)
                }
            }
        ).ShowModal()

    def delete_row(self):
        """Handle row delete."""

        row = self.m_plist_grid.GetGridCursorRow()
        self.m_plist_grid.DeleteRows(row, 1)
        self.m_plist_grid.GetParent().update_plist(sc.DELETE, {"table": "style", "index": row})

    def insert_row(self):
        """Handle inserting into row."""

        obj = {
            "name": "New Item",
            "scope": "comment",
            "settings": {
                "foreground": "#FFFFFF",
                "background": "#000000",
                "fontStyle": ""
            }
        }
        editor = self.GetParent().GetParent().GetParent()
        color_setting_dialog.ColorEditor(
            editor,
            obj,
            insert=True
        ).ShowModal()

    def row_up(self):
        """Handle row up."""

        grid = self.m_plist_grid
        row = grid.GetGridCursorRow()
        col = grid.GetGridCursorCol()
        if row > 0:
            text = [grid.GetCellValue(row, x) for x in range(0, 5)]
            bg = [grid.GetCellBackgroundColour(row, x) for x in range(0, 5)]
            fg = [grid.GetCellTextColour(row, x) for x in range(0, 5)]
            font = [grid.GetCellFont(row, x) for x in range(0, 5)]
            grid.DeleteRows(row, 1, False)
            grid.InsertRows(row - 1, 1, True)
            [grid.SetCellValue(row - 1, x, text[x]) for x in range(0, 5)]
            [grid.SetCellBackgroundColour(row - 1, x, bg[x]) for x in range(0, 5)]
            [grid.SetCellTextColour(row - 1, x, fg[x]) for x in range(0, 5)]
            [grid.SetCellFont(row - 1, x, font[x]) for x in range(0, 5)]
            self.go_cell(grid, row - 1, col, True)
            grid.GetParent().update_plist(sc.MOVE, {"from": row, "to": row - 1})
            grid.SetFocus()

    def row_down(self):
        """Handle row down."""

        grid = self.m_plist_grid
        row = grid.GetGridCursorRow()
        col = grid.GetGridCursorCol()
        if row < grid.GetNumberRows() - 1:
            text = [grid.GetCellValue(row, x) for x in range(0, 5)]
            bg = [grid.GetCellBackgroundColour(row, x) for x in range(0, 5)]
            fg = [grid.GetCellTextColour(row, x) for x in range(0, 5)]
            font = [grid.GetCellFont(row, x) for x in range(0, 5)]
            grid.DeleteRows(row, 1, False)
            grid.InsertRows(row + 1, 1, True)
            [grid.SetCellValue(row + 1, x, text[x]) for x in range(0, 5)]
            [grid.SetCellBackgroundColour(row + 1, x, bg[x]) for x in range(0, 5)]
            [grid.SetCellTextColour(row + 1, x, fg[x]) for x in range(0, 5)]
            [grid.SetCellFont(row + 1, x, font[x]) for x in range(0, 5)]
            self.go_cell(grid, row + 1, col, True)
            grid.GetParent().update_plist(sc.MOVE, {"from": row, "to": row + 1})
            grid.SetFocus()

    def is_fontstyle_cell(self):
        """Check if fontstyle cell."""

        return self.m_plist_grid.GetGridCursorCol() == 3

    def toggle_font_style(self, row, attr):
        """Toggle the font style."""

        # if not self.is_fontstyle_cell():
        #     return
        grid = self.m_plist_grid
        text = [grid.GetCellValue(row, x) for x in range(0, 5)]
        style = text[3].split(" ")
        try:
            idx = style.index(attr)
            del style[idx]
        except:
            style.append(attr)
        text[3] = " ".join(style)

        obj = {
            "name": text[0],
            "scope": text[4],
            "settings": {
                "foreground": text[1],
                "background": text[2],
                "fontStyle": text[3]
            }
        }
        grid.GetParent().update_row(row, obj)
        self.update_plist(sc.MODIFY, {"table": "style", "index": row, "data": obj})
        self.resize_table()

    def toggle_bold(self):
        """Toggle bold."""

        self.toggle_font_style(self.m_plist_grid.GetGridCursorRow(), "bold")

    def toggle_italic(self):
        """Toggle italic."""

        self.toggle_font_style(self.m_plist_grid.GetGridCursorRow(), "italic")

    def toggle_underline(self):
        """Toggle underline."""

        self.toggle_font_style(self.m_plist_grid.GetGridCursorRow(), "underline")

    def on_mouse_motion(self, event):
        """Handle mouse motion event."""

        self.mouse_motion(event)

    def on_edit_cell(self, event):
        """Handle editing cell event."""

        self.edit_cell()

    def on_grid_key_down(self, event):
        """Handle key down event on grid."""

        self.grid_key_down(event)

    def on_grid_select_cell(self, event):
        """Handle grid select event."""

        self.grid_select_cell(event)

    def on_row_up_click(self, event):
        """Handle row up click."""

        self.row_up()

    def on_row_down_click(self, event):
        """Handle row down click."""

        self.row_down()

    def on_row_add_click(self, event):
        """Handle row add click."""

        self.insert_row()

    def on_row_delete_click(self, event):
        """Handle row delete click."""

        self.delete_row()

    def on_grid_label_left_click(self, event):
        """Handle grid label left click."""

        return



"""Default theme."""
theme = {
    "name": "Grayscale",
    "settings": [
        {
            "settings": {
                "activeGuide": "#888888",
                "background": "#FFFFFF",
                "bracketContentsForeground": "#000000",
                "bracketContentsOptions": "underline",
                "bracketsForeground": "#000000",
                "bracketsOptions": "underline",
                "caret": "#000000",
                "findHighlight": "#666666",
                "findHighlightForeground": "#000000",
                "foreground": "#000000",
                "gutter": "#E5E5E5",
                "gutterForeground": "#000000",
                "inactiveSelection": "#888888",
                "invisibles": "#323232",
                "lineHighlight": "#2E2E2E22",
                "selection": "#666666",
                "selectionForeground": "#FFFFFF",
                "tagsOptions": "stippled_underline"
            }
        },
        {
            "name": "Comment",
            "scope": "comment",
            "settings": {
                "foreground": "#A5A5A5"
            }
        },
        {
            "name": "Storage type",
            "scope": "storage.type",
            "settings": {
                "fontStyle": "italic",
                "foreground": "#000000"
            }
        },
        {
            "name": "Class name",
            "scope": "entity.name.class",
            "settings": {
                "fontStyle": "underline",
                "foreground": "#000000"
            }
        },
        {
            "name": "Inherited class",
            "scope": "entity.other.inherited-class",
            "settings": {
                "fontStyle": "italic underline",
                "foreground": "#000000"
            }
        },
        {
            "name": "Function argument",
            "scope": "variable.parameter",
            "settings": {
                "fontStyle": "italic",
                "foreground": "#000000"
            }
        },
        {
            "name": "Library class/type",
            "scope": "support.type, support.class",
            "settings": {
                "fontStyle": "italic",
                "foreground": "#000000"
            }
        }
    ],
    "uuid": "07379361-ce49-4e6c-a03f-26378a7a2131"
}



"""
Custom Status Bar.

https://gist.github.com/facelessuser/5750045

Licensed under MIT
Copyright (c) 2013 - 2015 Isaac Muse <isaacmuse@gmail.com>
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions
of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""
from __future__ import unicode_literals
from collections import OrderedDict
import wx
import wx.lib.agw.supertooltip
from .. import util

if wx.VERSION > (2, 9, 4):  # When will this get fixed? :(
    def monkey_patch():
        """
        Monkey patch Supertooltips.

        Remove once WxPython gets its crap together.
        """

        import inspect
        import re

        target_line = re.compile(r'([ ]{8})(maxWidth = max\(bmpWidth\+\(textWidth\+self._spacing\*3\), maxWidth\)\n)')
        tt_source = inspect.getsourcelines(wx.lib.agw.supertooltip.ToolTipWindowBase.OnPaint)[0]
        count = 0
        found = False
        for line in tt_source:
            if not found:
                m = target_line.match(line)
                if m:
                    tt_source[count] = m.group(0)
                    found = True
                    count += 1
                    continue
            tt_source[count] = line[4:]
            count += 1
        exec(''.join(tt_source))
        wx.lib.agw.supertooltip.ToolTipWindowBase.OnPaint = locals()['OnPaint']  # noqa

    monkey_patch()


class ContextMenu(wx.Menu):
    """Context Menu."""

    def __init__(self, parent, menu, pos):
        """Attach the context menu to to the parent with the defined items."""

        wx.Menu.__init__(self)
        self._callbacks = {}

        for i in menu:
            menuid = wx.NewId()
            item = wx.MenuItem(self, menuid, i[0])
            self._callbacks[menuid] = i[1]
            self.Append(item)
            self.Bind(wx.EVT_MENU, self.on_callback, item)

        parent.PopupMenu(self, pos)

    def on_callback(self, event):
        """Execute the menu item callback."""

        menuid = event.GetId()
        self._callbacks[menuid](event)
        event.Skip()


class ToolTip(wx.lib.agw.supertooltip.SuperToolTip):
    """Tooltip."""

    def __init__(self, target, message, header="", style="Office 2007 Blue", start_delay=.1):
        """Attach the defined tooltip to the target."""

        super(ToolTip, self).__init__(message, header=header)
        self.SetTarget(target)
        self.ApplyStyle(style)
        self.SetStartDelay(start_delay)
        target.tooltip = self

    def hide(self):
        """Hide the tooltip."""

        if self._superToolTip:
            self._superToolTip.Destroy()


class TimedStatusExtension(object):
    """Timed status in status bar."""

    def set_timed_status(self, text, index=0):
        """
        Set the status for a short time.

        Save the previous status for restore
        when the timed status completes.
        """

        if self.text_timer[index].IsRunning():
            self.text_timer[index].Stop()
        else:
            self.saved_text = self.GetStatusText(index)
        self.SetStatusText(text, index)
        self.text_timer[index].Start(5000, oneShot=True)

    def sb_time_setup(self, field_count):
        """Setup timer for timed status."""

        self.field_count = field_count
        self.saved_text = [""] * field_count
        self.text_timer = [wx.Timer(self)] * field_count
        count = 0
        for x in self.text_timer:
            self.Bind(wx.EVT_TIMER, lambda event, index=count: self.clear_text(event, index), self.text_timer[count])
            count += 1

    def clear_text(self, event, index):
        """Clear the status."""

        self.SetStatusText(self.saved_text, index)

    def set_status(self, text, index=0):
        """Set the status."""

        if self.text_timer[index].IsRunning():
            self.text_timer[index].Stop()
        self.SetStatusText(text, index)


class IconTrayExtension(object):
    """Add icon tray extension."""

    fields = [-1]

    def sb_tray_setup(self):
        """Setup the status bar with icon tray."""

        self.SetFieldsCount(len(self.fields) + 1)
        self.SetStatusText('', 0)
        self.SetStatusWidths(self.fields + [1])
        self.sb_icons = OrderedDict()
        self.Bind(wx.EVT_SIZE, self.on_sb_size)

    def remove_icon(self, name):
        """Remove an icon from the tray."""

        if name in self.sb_icons:
            self.hide_tooltip(name)
            self.sb_icons[name].Destroy()
            del self.sb_icons[name]
            self.place_icons(resize=True)

    def hide_tooltip(self, name):
        """Hide the tooltip."""

        if self.sb_icons[name].tooltip:
            self.sb_icons[name].tooltip.hide()

    def set_icon(
        self, name, icon, msg=None, context=None,
        click_right=None, click_left=None,
        dclick_right=None, dclick_left=None
    ):
        """
        Set the given icon in the tray.

        Attach a menu and/or tooltip if provided.
        """

        if name in self.sb_icons:
            self.hide_tooltip(name)
            self.sb_icons[name].Destroy()
        self.sb_icons[name] = wx.StaticBitmap(self, label=icon)
        if msg is not None:
            ToolTip(self.sb_icons[name], msg)
        if click_left is not None:
            self.sb_icons[name].Bind(wx.EVT_LEFT_DOWN, click_left)
        if context is not None:
            self.sb_icons[name].Bind(wx.EVT_RIGHT_DOWN, lambda e: self.show_menu(name, context))
        elif click_right is not None:
            self.sb_icons[name].Bind(wx.EVT_RIGHT_DOWN, click_right)
        if dclick_left is not None:
            self.sb_icons[name].Bind(wx.EVT_LEFT_DCLICK, dclick_left)
        if dclick_right is not None:
            self.sb_icons[name].Bind(wx.EVT_RIGHT_DCLICK, dclick_right)
        self.place_icons(resize=True)

    def show_menu(self, name, context):
        """Show context menu on icon in tray."""

        self.hide_tooltip(name)
        ContextMenu(self, context, self.sb_icons[name].GetPosition())

    def place_icons(self, resize=False):
        """Calculate new icon position and icon tray size."""

        x_offset = 0
        if resize:
            platform = util.platform()
            if platform in "osx":
                # OSX must increment by 10
                self.SetStatusWidths([-1, len(self.sb_icons) * 20 + 10])
            elif platform == "windows":
                # In at least wxPython 2.9+, the first icon inserted changes the size, additional icons don't.
                # I've only tested >= 2.9.
                if len(self.sb_icons):
                    self.SetStatusWidths([-1, (len(self.sb_icons) - 1) * 20 + 1])
                else:
                    self.SetStatusWidths([-1, len(self.sb_icons) * 20 + 1])
            else:
                # Linux? Should be fine with 1, but haven't tested yet.
                self.SetStatusWidths([-1, len(self.sb_icons) * 20 + 1])
        rect = self.GetFieldRect(len(self.fields))
        for v in self.sb_icons.values():
            v.SetPosition((rect.x + x_offset, rect.y))
            v.Hide()
            v.Show()
            x_offset += 20

    def on_sb_size(self, event):
        """Ensure icons are properly placed on resize."""

        event.Skip()
        self.place_icons()


class CustomStatusExtension(IconTrayExtension, TimedStatusExtension):
    """Custom status extension."""

    def sb_setup(self, fields):
        """Setup the extention variant of the CustomStatusBar object."""

        self.fields = fields
        self.sb_tray_setup()
        self.sb_time_setup(len(self.fields))


class CustomStatusBar(wx.StatusBar, CustomStatusExtension):
    """Custom status bar."""

    def __init__(self, parent, name, fields=None):
        """Init the CustomStatusBar object."""

        field_array = [-1] if not fields else fields[:]
        super(CustomStatusBar, self).__init__(
            parent,
            id=wx.ID_ANY,
            style=wx.STB_DEFAULT_STYLE,
            name=name
        )
        self.sb_setup(field_array)


def extend(instance, extension):
    """Extend instance with extension class."""

    instance.__class__ = type(
        ('%s_extended_with_%s' if util.PY3 else b'%s_extended_with_%s') % (
            instance.__class__.__name__, extension.__name__
        ),
        (instance.__class__, extension),
        {}
    )


def extend_sb(sb, fields=None):
    """Extend the statusbar."""

    field_array = [-1] if not fields else fields[:]
    extend(sb, CustomStatusExtension)
    sb.sb_setup(field_array)



"""Basic dialogs."""
from __future__ import unicode_literals
from . import messages


def filepicker(msg, default_path, wildcard, save=False):
    """Call file picker."""

    return messages.filepickermsg(msg, default_path, wildcard, save)


def yesno(question, title='Yes or no?', bitmap=None, yes="Okay", no="Cancel"):
    """Prompt for yes/no."""

    return messages.promptmsg(question, title, bitmap, yes, no)


def infomsg(msg, title="INFO", bitmap=None):
    """Info message."""

    messages.infomsg(msg, title, bitmap)


def errormsg(msg, title="ERROR", bitmap=None):
    """Error message."""

    messages.errormsg(msg, title, bitmap)


def warnmsg(msg, title="WARNING", bitmap=None):
    """Warning message."""

    messages.warnmsg(msg, title, bitmap)



"""
About Dialog.

Licensed under MIT
Copyright (c) 2013 - 2015 Isaac Muse <isaacmuse@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions
of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""
from __future__ import unicode_literals
import wx
from . import gui
from . import data
from .. import __meta__


class AboutDialog(gui.AboutDialog):
    """About Dialog."""

    def __init__(self, parent):
        """Initialize the AboutDialog object."""

        super(AboutDialog, self).__init__(parent)

        self.SetTitle("About")

        self.m_bitmap = wx.StaticBitmap(
            self.m_about_panel,
            wx.ID_ANY,
            data.get_bitmap('subclrschm_dialog.png'),
            wx.DefaultPosition,
            wx.Size(64, 64), 0
        )
        self.m_app_label.SetLabel(__meta__.__app__)
        self.m_version_label.SetLabel(
            "Version: %s %s" % (__meta__.__version__, __meta__.__status__)
        )
        self.m_developers_label.SetLabel(
            "Developer(s):\n%s" % ("\n".join(["    %s - %s" % (m[0], m[1]) for m in __meta__.__maintainers__]))
        )

        self.m_dev_toggle.SetLabel("Contact" + " >>")

        self.Fit()

    def on_toggle(self, event):
        """Show/hide contact info on when contact button is toggled."""

        if self.m_dev_toggle.GetValue():
            self.m_dev_toggle.SetLabel("Contact" + " <<")
            self.m_developers_label.Show()
        else:
            self.m_dev_toggle.SetLabel("Contact" + " >>")
            self.m_developers_label.Hide()
        self.Fit()
        self.Refresh()



"""Spell check with aspell."""
from __future__ import unicode_literals
import subprocess
import os
import sys
import yaml
import codecs

PY3 = sys.version_info >= (3, 0)

USER_DICT = '.dictionary'
BUILD_DIR = os.path.join('.', 'build', 'docs')
MKDOCS_CFG = 'mkdocs.yml'
COMPILED_DICT = os.path.join(BUILD_DIR, 'dictionary.bin')
MKDOCS_SPELL = os.path.join(BUILD_DIR, MKDOCS_CFG)
MKDOCS_BUILD = os.path.join(BUILD_DIR, 'site')


def console(cmd, input_file=None):
    """Call with arguments."""

    returncode = None
    output = None

    if sys.platform.startswith('win'):
        startupinfo = subprocess.STARTUPINFO()
        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        process = subprocess.Popen(
            cmd,
            startupinfo=startupinfo,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            stdin=subprocess.PIPE,
            shell=False
        )
    else:
        process = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            stdin=subprocess.PIPE,
            shell=False
        )

    if input_file is not None:
        with open(input_file, 'rb') as f:
            process.stdin.write(f.read())
    output = process.communicate()
    returncode = process.returncode

    assert returncode == 0, "Runtime Error: %s" % (
        output[0].rstrip().decode('utf-8') if PY3 else output[0]
    )

    return output[0].decode('utf-8') if PY3 else output[0]


def yaml_dump(data, stream=None, dumper=yaml.Dumper, **kwargs):
    """Special dumper wrapper to modify the yaml dumper."""

    class Dumper(dumper):
        """Custom dumper."""

    if not PY3:
        # Unicode
        Dumper.add_representer(
            unicode,  # noqa
            lambda dumper, value: dumper.represent_scalar(u'tag:yaml.org,2002:str', value)
        )

    return yaml.dump(data, stream, Dumper, **kwargs)


def yaml_load(source, loader=yaml.Loader):
    """
    Wrap PyYaml's loader so we can extend it to suit our needs.

    Load all strings as unicode: http://stackoverflow.com/a/2967461/3609487.
    """

    def construct_yaml_str(self, node):
        """Override the default string handling function to always return Unicode objects."""
        return self.construct_scalar(node)

    class Loader(loader):
        """Define a custom loader to leave the global loader unaltered."""

    # Attach our unicode constructor to our custom loader ensuring all strings
    # will be unicode on translation.
    Loader.add_constructor('tag:yaml.org,2002:str', construct_yaml_str)

    return yaml.load(source, Loader)


def patch_doc_config(config_file):
    """Patch the config file to wrap arithmatex with a tag aspell can ignore."""

    with open(config_file, 'rb') as f:
        config = yaml_load(f)

    index = 0
    for extension in config.get('markdown_extensions', []):
        if isinstance(extension, str if PY3 else unicode) and extension == 'pymdownx.arithmatex':  # noqa
            config['markdown_extensions'][index] = {'pymdownx.arithmatex': {'insert_as_script': True}}
            break
        elif isinstance(extension, dict) and 'pymdownx.arithmatex' in extension:
            if isinstance(extension['pymdownx.arithmatex'], dict):
                extension['pymdownx.arithmatex']['insert_as_script'] = True
            elif extension['pymdownx.arithmatex'] is None:
                extension['pymdownx.arithmatex'] = {'insert_as_script': True}
            break
        index += 1

    with codecs.open(MKDOCS_SPELL, "w", encoding="utf-8") as f:
        yaml_dump(
            config, f,
            width=None,
            indent=4,
            allow_unicode=True,
            default_flow_style=False
        )
    return MKDOCS_SPELL


def build_docs():
    """Build docs with MkDocs."""
    print('Building Docs...')
    print(
        console(
            [
                sys.executable,
                '-m', 'mkdocs', 'build', '--clean',
                '-d', MKDOCS_BUILD,
                '-f', patch_doc_config(MKDOCS_CFG)
            ]
        )
    )


def compile_dictionary():
    """Compile user dictionary."""
    if os.path.exists(COMPILED_DICT):
        os.remove(COMPILED_DICT)
    print("Compiling Custom Dictionary...")
    print(
        console(
            [
                'aspell',
                '--lang=en',
                '--encoding=utf-8',
                'create',
                'master',
                COMPILED_DICT
            ],
            USER_DICT
        )
    )


def check_spelling():
    """Check spelling."""
    print('Spell Checking...')

    fail = False

    for base, dirs, files in os.walk(MKDOCS_BUILD):
        # Remove child folders based on exclude rules
        for f in files:
            if f.endswith('.html'):
                file_name = os.path.join(base, f)
                wordlist = console(
                    [
                        'aspell',
                        'list',
                        '--lang=en',
                        '--mode=html',
                        '--encoding=utf-8',
                        '--add-html-skip=code',
                        '--add-html-skip=pre',
                        '--add-html-skip=nospell',
                        '--extra-dicts=%s' % COMPILED_DICT
                    ],
                    file_name
                )

                words = [w for w in sorted(set(wordlist.split('\n'))) if w]

                if words:
                    fail = True
                    print('Misspelled words in %s' % file_name)
                    print('-' * 80)
                    for word in words:
                        print(word)
                    print('-' * 80)
                    print('\n')
    return fail


def main():
    """Main."""
    if not os.path.exists(BUILD_DIR):
        os.makedirs(BUILD_DIR)
    build_docs()
    compile_dictionary()
    return check_spelling()


if __name__ == "__main__":
    sys.exit(main())



"""Compatibility module."""
from __future__ import unicode_literals
import sys
import functools
import os
import plistlib
import copy
import locale

PY3 = (3, 0) <= sys.version_info
PY2 = (2, 0) <= sys.version_info < (3, 0)

if sys.platform.startswith('win'):
    _PLATFORM = "windows"
elif sys.platform == "darwin":
    _PLATFORM = "osx"
else:
    _PLATFORM = "linux"

if PY3:
    string_type = str
    ustr = str
    bstr = bytes
    CommonBrokenPipeError = BrokenPipeError  # noqa
else:
    string_type = basestring
    ustr = unicode
    bstr = str

    class CommonBrokenPipeError(Exception):
        """
        Broken Pipe Error.

        Include this for consistency, but we won't actually
        capture this in PY2.
        """


def platform():
    """Get Platform."""

    return _PLATFORM


def sorted_callback(l, sorter):
    """Use a callback with sort in a PY2/PY3 way."""

    if PY3:
        l.sort(key=functools.cmp_to_key(sorter))
    else:
        l.sort(sorter)


def to_ascii_bytes(string):
    """Convert unicode to ascii byte string."""

    return bytes(string, 'ascii') if PY3 else bytes(string)


def to_ustr(obj):
    """Convert to string."""

    if isinstance(obj, ustr):
        return obj
    elif isinstance(obj, bstr):
        return ustr(obj, 'utf-8')
    else:
        return ustr(obj)


def to_bstr(obj):
    """Convert to byte string."""

    assert isinstance(obj, string_type), TypeError
    return obj.encode('utf-8') if isinstance(obj, ustr) else obj


def getcwd():
    """Get the current working directory."""

    if PY3:
        return os.getcwd()
    else:
        return os.getcwdu()


def iternext(item):
    """Iterate to next."""

    if PY3:
        return item.__next__()
    else:
        return item.next()


def translate(lang, text):
    """Translate text."""

    return lang.gettext(text) if PY3 else lang.ugettext(text)


def dump_plist(content):
    """Dump plist."""

    return plistlib.dumps(content).decode('utf-8') if PY3 else plistlib.writePlistToString(content).decode('utf-8')


def to_unicode_argv():
    """Convert inputs to Unicode."""

    args = copy.copy(sys.argv)

    if PY2:
        if _PLATFORM == "windows":
            # Solution copied from http://stackoverflow.com/a/846931/145400

            from ctypes import POINTER, byref, cdll, c_int, windll
            from ctypes.wintypes import LPCWSTR, LPWSTR

            GetCommandLineW = cdll.kernel32.GetCommandLineW
            GetCommandLineW.argtypes = []
            GetCommandLineW.restype = LPCWSTR

            CommandLineToArgvW = windll.shell32.CommandLineToArgvW
            CommandLineToArgvW.argtypes = [LPCWSTR, POINTER(c_int)]
            CommandLineToArgvW.restype = POINTER(LPWSTR)

            cmd = GetCommandLineW()
            argc = c_int(0)
            argv = CommandLineToArgvW(cmd, byref(argc))
            if argc.value > 0:
                # Remove Python executable and commands if present
                start = argc.value - len(sys.argv)
                args = [argv[i] for i in xrange(start, argc.value)]
        else:
            cli_encoding = sys.stdin.encoding or locale.getpreferredencoding()
            args = [arg.decode(cli_encoding) for arg in sys.argv if isinstance(arg, bstr)]
    return args



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Setup package."""
from setuptools import setup, find_packages
import os
import imp
import traceback


def get_version():
    """Get version and version_info without importing the entire module."""

    devstatus = {
        'alpha': '3 - Alpha',
        'beta': '4 - Beta',
        'candidate': '4 - Beta',
        'final': '5 - Production/Stable'
    }
    path = os.path.join(os.path.dirname(__file__), 'subclrschm', 'lib')
    fp, pathname, desc = imp.find_module('__version__', [path])
    try:
        v = imp.load_module('__version__', fp, pathname, desc)
        return v.version, devstatus[v.version_info[3]]
    except Exception:
        print(traceback.format_exc())
    finally:
        fp.close()


VER, DEVSTATUS = get_version()

LONG_DESC = '''
Subclrschm is a color scheme editor for Sublime Text 3.

It is built with wxPython 4.0.0+ and requires Python 2.7 or 3.4+.
You can learn more about using Rummage by `reading the docs`_.

.. _`reading the docs`: http://facelessuser.github.io/subclrschm/

Support
=======

Help and support is available here at the repository's `bug tracker`_.
Please read about `support and contributing`_ before creating issues.

.. _`bug tracker`: https://github.com/facelessuser/subclrschm/issues
.. _`support and contributing`: http://facelessuser.github.io/subclrschm/contributing/
'''

setup(
    name='subclrschm',
    version=VER,
    keywords='Sublime color scheme',
    description='GUI for editing Sublime Text color schemes.',
    long_description=LONG_DESC,
    author='Isaac Muse',
    author_email='Isaac.Muse@gmail.com',
    url='https://github.com/facelessuser/subclrschm',
    packages=find_packages(exclude=['tests', 'tools']),
    install_requires=[
        "wxpython>=4.0.0a3"
    ],
    zip_safe=False,
    entry_points={
        'gui_scripts': [
            'subclrschm=subclrschm.__main__:main'
        ]
    },
    license='MIT License',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



"""Script to patch wxFormBuilder output of wxPython files for version 4.0."""
from __future__ import print_function
import re
import os
import codecs

RE_ADD_WX_ADV = re.compile(r'(import wx)\n')
RE_SIZE_HINTS_SZ = re.compile(r'\bSetSizeHintsSz\b')
RE_ADD_SPACER = re.compile(r'\bAddSpacer\b')
RE_APPEND_ITEM = re.compile(r'\bAppendItem\b')
RE_ADV = re.compile(r'\b(wx\.)(GenericDatePickerCtrl|DP_)')
RE_ST = re.compile(r'\b(wx\.ST)(_)')

gui = os.path.join('subclrschm', 'lib', 'gui', 'gui.py')

with codecs.open(gui, 'r', encoding="utf-8") as f:
    buf = f.read().replace('\r', '')

    buf = RE_ADD_WX_ADV.sub(r'\1\nimport wx.adv\n', buf)
    buf = RE_SIZE_HINTS_SZ.sub('SetSizeHints', buf)
    buf = RE_ADD_SPACER.sub('Add', buf)
    buf = RE_APPEND_ITEM.sub('Append', buf)
    buf = RE_ADV.sub(r'\1adv.\2', buf)
    buf = RE_ST.sub(r'\1B\2', buf)

with codecs.open(gui, 'w', encoding='utf-8') as f:
    f.write(buf.strip() + '\n')



"""
RGBA.

Licensed under MIT
Copyright (c) 2012 - 2016 Isaac Muse <isaacmuse@gmail.com>
"""
import re
from colorsys import rgb_to_hls, hls_to_rgb, rgb_to_hsv, hsv_to_rgb
import decimal

RGB_CHANNEL_SCALE = 1.0 / 255.0
HUE_SCALE = 1.0 / 360.0


def clamp(value, mn, mx):
    """Clamp the value to the the given minimum and maximum."""

    return max(min(value, mx), mn)


def round_int(dec):
    """Round float to nearest int using expected rounding."""

    return int(decimal.Decimal(dec).quantize(decimal.Decimal('0'), decimal.ROUND_HALF_UP))


class RGBA(object):
    """RGBA object for converting between color formats or applying filters to the color."""

    r = None
    g = None
    b = None
    a = None
    color_pattern = re.compile(r"^#(?:([A-Fa-f\d]{6})([A-Fa-f\d]{2})?|([A-Fa-f\d]{3}))")

    def __init__(self, s=None):
        """Initialize."""

        if s is None:
            s = "#000000FF"
        self.r, self.g, self.b, self.a = self._split_channels(s)

    def _split_channels(self, s):
        """Split the color into color channels: red, green, blue, alpha."""

        def alpha_channel(alpha):
            """Get alpha channel."""
            return int(alpha, 16) if alpha else 0xFF

        m = self.color_pattern.match(s)
        assert(m is not None)
        if m.group(1):
            return int(s[1:3], 16), int(s[3:5], 16), int(s[5:7], 16), alpha_channel(m.group(2))
        else:
            return int(s[1] * 2, 16), int(s[2] * 2, 16), int(s[3] * 2, 16), 0xFF

    def get_rgba(self):
        """Get the RGB color with the alpha channel."""

        return "#%02X%02X%02X%02X" % (self.r, self.g, self.b, self.a)

    def get_rgb(self):
        """Get the RGB valuie."""

        return "#%02X%02X%02X" % (self.r, self.g, self.b)

    def apply_alpha(self, background="#000000FF"):
        """
        Apply the given transparency with the given background.

        This gives a color that represents what the eye sees with
        the transparent color against the given background.
        """

        def tx_alpha(cf, af, cb, ab):
            """Translate the color channel with the alpha channel and background channel color."""

            return round_int(
                abs(
                    cf * (af * RGB_CHANNEL_SCALE) + cb * (ab * RGB_CHANNEL_SCALE) * (1 - (af * RGB_CHANNEL_SCALE))
                )
            ) & 0xFF

        if self.a < 0xFF:
            r, g, b, a = self._split_channels(background)

            self.r = tx_alpha(self.r, self.a, r, a)
            self.g = tx_alpha(self.g, self.a, g, a)
            self.b = tx_alpha(self.b, self.a, b, a)

        return self.get_rgb()

    def get_luminance(self):
        """Get percieved luminance."""

        return clamp(round_int(0.299 * self.r + 0.587 * self.g + 0.114 * self.b), 0, 255)

    def get_true_luminance(self):
        """Get true liminance."""

        l = self.tohls()[1]
        return clamp(round_int(l * 255.0), 0, 255)

    def alpha(self, factor):
        """Adjust alpha."""

        self.a = round_int(clamp(self.a + (255.0 * factor) - 255.0, 0.0, 255.0))

    def red(self, factor):
        """Adjust red."""

        self.r = round_int(clamp(self.r + (255.0 * factor) - 255.0, 0.0, 255.0))

    def green(self, factor):
        """Adjust green."""

        self.g = round_int(clamp(self.g + (255.0 * factor) - 255.0, 0.0, 255.0))

    def blue(self, factor):
        """Adjust blue."""

        self.b = round_int(clamp(self.b + (255.0 * factor) - 255.0, 0.0, 255.0))

    def luminance(self, factor):
        """Get true luminance."""

        h, l, s = self.tohls()
        l = clamp(l + factor - 1.0, 0.0, 1.0)
        self.fromhls(h, l, s)

    def tohsv(self):
        """Convert to HSV color format."""

        return rgb_to_hsv(self.r * RGB_CHANNEL_SCALE, self.g * RGB_CHANNEL_SCALE, self.b * RGB_CHANNEL_SCALE)

    def fromhsv(self, h, s, v):
        """Convert to RGB from HSV."""

        r, g, b = hsv_to_rgb(h, s, v)
        self.r = round_int(r * 255.0) & 0xFF
        self.g = round_int(g * 255.0) & 0xFF
        self.b = round_int(b * 255.0) & 0xFF

    def tohls(self):
        """Convert to HLS color format."""

        return rgb_to_hls(self.r * RGB_CHANNEL_SCALE, self.g * RGB_CHANNEL_SCALE, self.b * RGB_CHANNEL_SCALE)

    def fromhls(self, h, l, s):
        """Convert to RGB from HSL."""

        r, g, b = hls_to_rgb(h, l, s)
        self.r = round_int(r * 255.0) & 0xFF
        self.g = round_int(g * 255.0) & 0xFF
        self.b = round_int(b * 255.0) & 0xFF

    def tohwb(self):
        """Convert to HWB from RGB."""

        h, s, v = self.tohsv()
        w = (1.0 - s) * v
        b = 1.0 - v
        return h, w, b

    def fromhwb(self, h, w, b):
        """Convert to RGB from HWB."""

        # Normalize white and black
        # w + b <= 1.0
        if w + b > 1.0:
            norm_factor = 1.0 / (w + b)
            w *= norm_factor
            b *= norm_factor

        # Convert to HSV and then to RGB
        s = 1.0 - (w / (1.0 - b))
        v = 1.0 - b
        r, g, b = hsv_to_rgb(h, s, v)
        self.r = round_int(r * 255.0) & 0xFF
        self.g = round_int(g * 255.0) & 0xFF
        self.b = round_int(b * 255.0) & 0xFF

    def colorize(self, deg):
        """Colorize the color with the given hue."""

        h, l, s = self.tohls()
        h = clamp(deg * HUE_SCALE, 0.0, 1.0)
        self.fromhls(h, l, s)

    def hue(self, deg):
        """Shift the hue."""

        d = deg * HUE_SCALE
        h, l, s = self.tohls()
        h = h + d
        while h > 1.0:
            h -= 1.0
        while h < 0.0:
            h += 1.0
        self.fromhls(h, l, s)

    def invert(self):
        """Invert the color."""

        self.r ^= 0xFF
        self.g ^= 0xFF
        self.b ^= 0xFF

    def saturation(self, factor):
        """Saturate or unsaturate the color by the given factor."""

        h, l, s = self.tohls()
        s = clamp(s + factor - 1.0, 0.0, 1.0)
        self.fromhls(h, l, s)

    def grayscale(self):
        """Convert the color with a grayscale filter."""

        luminance = self.get_luminance() & 0xFF
        self.r = luminance
        self.g = luminance
        self.b = luminance

    def sepia(self):
        """Apply a sepia filter to the color."""

        r = clamp(round_int((self.r * .393) + (self.g * .769) + (self.b * .189)), 0, 255) & 0xFF
        g = clamp(round_int((self.r * .349) + (self.g * .686) + (self.b * .168)), 0, 255) & 0xFF
        b = clamp(round_int((self.r * .272) + (self.g * .534) + (self.b * .131)), 0, 255) & 0xFF
        self.r, self.g, self.b = r, g, b

    def _get_overage(self, c):
        """Get overage."""

        if c < 0.0:
            o = 0.0 + c
            c = 0.0
        elif c > 255.0:
            o = c - 255.0
            c = 255.0
        else:
            o = 0.0
        return o, c

    def _distribute_overage(self, c, o, s):
        """Distribute overage."""

        channels = len(s)
        if channels == 0:
            return c
        parts = o / len(s)
        if "r" in s and "g" in s:
            c = c[0] + parts, c[1] + parts, c[2]
        elif "r" in s and "b" in s:
            c = c[0] + parts, c[1], c[2] + parts
        elif "g" in s and "b" in s:
            c = c[0], c[1] + parts, c[2] + parts
        elif "r" in s:
            c = c[0] + parts, c[1], c[2]
        elif "g" in s:
            c = c[0], c[1] + parts, c[2]
        else:  # "b" in s:
            c = c[0], c[1], c[2] + parts
        return c

    def brightness(self, factor):
        """
        Adjust the brightness by the given factor.

        Brightness is determined by percieved luminance.
        """

        channels = ["r", "g", "b"]
        total_lumes = clamp(self.get_luminance() + (255.0 * factor) - 255.0, 0.0, 255.0)

        if total_lumes == 255.0:
            # white
            self.r, self.g, self.b = 0xFF, 0xFF, 0xFF
        elif total_lumes == 0.0:
            # black
            self.r, self.g, self.b = 0x00, 0x00, 0x00
        else:
            # Adjust Brightness
            pts = (total_lumes - 0.299 * self.r - 0.587 * self.g - 0.114 * self.b)
            slots = set(channels)
            components = [float(self.r) + pts, float(self.g) + pts, float(self.b) + pts]
            count = 0
            for c in channels:
                overage, components[count] = self._get_overage(components[count])
                if overage:
                    slots.remove(c)
                    components = list(self._distribute_overage(components, overage, slots))
                count += 1

            self.r = clamp(round_int(components[0]), 0, 255) & 0xFF
            self.g = clamp(round_int(components[1]), 0, 255) & 0xFF
            self.b = clamp(round_int(components[2]), 0, 255) & 0xFF



"""Color setting dialog."""
from __future__ import unicode_literals
import wx
from . import gui
from . import settings_key_bindings
from . import settings_codes as sc
from ..rgba import RGBA


class ColorEditor(gui.ColorSetting, settings_key_bindings.SettingsKeyBindings):
    """Color editor."""

    def __init__(self, parent, obj, insert=False):
        """Initialize."""

        super(ColorEditor, self).__init__(parent)
        self.setup_keybindings()
        self.Fit()
        size = self.GetSize()
        self.SetMinSize(size)
        size.Set(-1, size[1])
        self.SetMaxSize(size)
        self.foreground_save = ""
        self.background_save = ""
        self.apply_settings = False
        self.color_obj = obj
        self.insert = bool(insert)

        self.m_bold_checkbox.SetValue(False)
        self.m_italic_checkbox.SetValue(False)
        self.m_underline_checkbox.SetValue(False)

        for x in self.color_obj["settings"]["fontStyle"].split(" "):
            if x == "bold":
                self.m_bold_checkbox.SetValue(True)
            elif x == "italic":
                self.m_italic_checkbox.SetValue(True)
            elif x == "underline":
                self.m_underline_checkbox.SetValue(True)

        self.m_name_textbox.SetValue(self.color_obj["name"])
        self.m_scope_textbox.SetValue(self.color_obj["scope"])

        self.m_foreground_textbox.SetValue(self.color_obj["settings"]["foreground"])
        if self.color_obj["settings"]["foreground"] == "":
            cl = RGBA("#FFFFFF")
            bg = wx.Colour(cl.r, cl.g, cl.b)
            self.m_foreground_picker.SetBackgroundColour(bg)
            if cl.get_luminance() > 128:
                fg = wx.Colour(0, 0, 0)
            else:
                fg = wx.Colour(255, 255, 255)
            self.m_foreground_button_label.SetForegroundColour(fg)

        self.m_background_textbox.SetValue(self.color_obj["settings"]["background"])
        if self.color_obj["settings"]["background"] == "":
            cl = RGBA("#FFFFFF")
            bg = wx.Colour(cl.r, cl.g, cl.b)
            self.m_background_picker.SetBackgroundColour(bg)
            if cl.get_luminance() > 128:
                fg = wx.Colour(0, 0, 0)
            else:
                fg = wx.Colour(255, 255, 255)
            self.m_background_button_label.SetForegroundColour(fg)

    def on_foreground_button_click(self, event):
        """Handle foreground button click event."""

        color = None
        data = wx.ColourData()
        data.SetChooseFull(True)

        alpha = None

        text = self.m_foreground_textbox.GetValue()
        if text == "":
            rgb = RGBA("#FFFFFF")
        else:
            rgb = RGBA(text)
            if len(text) == 9:
                alpha == text[7:9]

        # set the default color in the chooser
        data.SetColour(wx.Colour(rgb.r, rgb.g, rgb.b))

        # construct the chooser
        dlg = wx.ColourDialog(self, data)

        if dlg.ShowModal() == wx.ID_OK:
            # set the panel background color
            color = dlg.GetColourData().GetColour().GetAsString(wx.C2S_HTML_SYNTAX)
            self.m_foreground_textbox.SetValue(color if alpha is None else color + alpha)
        dlg.Destroy()
        event.Skip()

    def on_background_button_click(self, event):
        """Handle background button click event."""

        color = None
        data = wx.ColourData()
        data.SetChooseFull(True)

        alpha = None

        text = self.m_background_textbox.GetValue()
        if text == "":
            rgb = RGBA("#FFFFFF")
        else:
            rgb = RGBA(text)
            if len(text) == 9:
                alpha == text[7:9]

        # set the default color in the chooser
        data.SetColour(wx.Colour(rgb.r, rgb.g, rgb.b))

        # construct the chooser
        dlg = wx.ColourDialog(self, data)

        if dlg.ShowModal() == wx.ID_OK:
            # set the panel background color
            color = dlg.GetColourData().GetColour().GetAsString(wx.C2S_HTML_SYNTAX)
            self.m_background_textbox.SetValue(color if alpha is None else color + alpha)
        dlg.Destroy()
        event.Skip()

    def on_background_change(self, event):
        """On background change event."""

        text = self.m_background_textbox.GetValue()
        try:
            if text == "":
                cl = RGBA("#FFFFFF")
            else:
                cl = RGBA(text)
        except:
            event.Skip()
            return

        cl.apply_alpha(self.Parent.m_style_settings.bg_color.get_rgb())
        bg = wx.Colour(cl.r, cl.g, cl.b)
        self.m_background_picker.SetBackgroundColour(bg)
        if cl.get_luminance() > 128:
            fg = wx.Colour(0, 0, 0)
        else:
            fg = wx.Colour(255, 255, 255)
        self.m_background_button_label.SetForegroundColour(fg)
        self.m_background_picker.Refresh()

    def on_foreground_change(self, event):
        """Handle foreground change event."""

        text = self.m_foreground_textbox.GetValue()
        try:
            if text == "":
                cl = RGBA("#FFFFFF")
            else:
                cl = RGBA(text)
        except:
            event.Skip()
            return

        cl.apply_alpha(self.Parent.m_style_settings.bg_color.get_rgb())
        bg = wx.Colour(cl.r, cl.g, cl.b)
        self.m_foreground_picker.SetBackgroundColour(bg)
        if cl.get_luminance() > 128:
            fg = wx.Colour(0, 0, 0)
        else:
            fg = wx.Colour(255, 255, 255)
        self.m_foreground_button_label.SetForegroundColour(fg)
        self.m_foreground_picker.Refresh()

    def on_foreground_focus(self, event):
        """Handle foreground focus event."""
        self.foreground_save = self.m_foreground_textbox.GetValue()
        event.Skip()

    def on_background_focus(self, event):
        """Handle background focus event."""
        self.background_save = self.m_background_textbox.GetValue()
        event.Skip()

    def on_foreground_blur(self, event):
        """Handle foreground blur event."""
        text = self.m_foreground_textbox.GetValue()
        if text != "":
            try:
                RGBA(text)
            except:
                self.m_foreground_textbox.SetValue(self.foreground_save)
        event.Skip()

    def on_background_blur(self, event):
        """Handle background blur event."""
        text = self.m_background_textbox.GetValue()
        if text != "":
            try:
                RGBA(text)
            except:
                self.m_background_textbox.SetValue(self.background_save)
        event.Skip()

    def on_apply_button_click(self, event):
        """Handle appply button click event."""

        self.apply_settings = True
        self.Close()

    def on_set_color_close(self, event):
        """Handle set color close event."""

        fontstyle = []
        if self.m_bold_checkbox.GetValue():
            fontstyle.append("bold")
        if self.m_italic_checkbox.GetValue():
            fontstyle.append("italic")
        if self.m_underline_checkbox.GetValue():
            fontstyle.append("underline")

        if self.apply_settings:
            self.color_obj = {
                "name": self.m_name_textbox.GetValue(),
                "scope": self.m_scope_textbox.GetValue(),
                "settings": {
                    "foreground": self.m_foreground_textbox.GetValue(),
                    "background": self.m_background_textbox.GetValue(),
                    "fontStyle": " ".join(fontstyle)
                }
            }

            if self.insert:
                grid = self.Parent.m_style_settings.m_plist_grid
                num = grid.GetNumberRows()
                row = grid.GetGridCursorRow()
                if num > 0:
                    grid.InsertRows(row, 1, True)
                else:
                    grid.AppendRows(1)
                    row = 0
                grid.GetParent().update_row(row, self.color_obj)
                grid.GetParent().go_cell(grid, row, 0)
                self.Parent.update_plist(sc.JSON_ADD, {"table": "style", "index": row, "data": self.color_obj})
            else:
                self.Parent.set_style_object(self.color_obj)
        event.Skip()



"""Settings key bindings."""
from __future__ import unicode_literals
import wx


class SettingsKeyBindings(object):
    """Key binding for settings."""

    def setup_keybindings(self):
        """Setup the key bindings."""
        self.Bind(wx.EVT_CHAR_HOOK, self.on_char_hook)

    def on_char_hook(self, event):
        """Evaluate keycode on char hook."""

        if event.GetKeyCode() == wx.WXK_ESCAPE:
            self.Close()
        event.Skip()
